****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidacov.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 12/09/2016
* Version: 1.14
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will manage covariates.                                        
*   
*  Program inputs:  
*	-DPLocal.&RUNID._mstr
*	-infolder.&COHORTFILE. 
*	-infolder.&UTILFILE.
*	-infolder.Comorbcodes
*	-infolder.&Covariatecodes.
*	-infolder.&Type2File.
*	-infolder.&DRUGCLASSFILE.
*	-infolder.&MONITORINGFILE.
*                                     
*  Program outputs: 
*	-DPLocal.&RUNID._ads_psmatch
*	-DPLocal.&RUNID._Claims_icddx
*	-DPLocal.&RUNID._Claims_icdpx
*	-DPLocal.&RUNID._Claims_cpt
*	-DPLocal.&RUNID._Claims_hcpcs
*	-DPLocal.&RUNID._Claims_drugclass 
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*	-This macro includes the optional execution of ms_util and ms_cci_elix macros                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*     1.3    03/31/2015    AC       Made changes to programming notes in header section.
*
*     1.4    03/31/2015    EM       Adapt code to Surveilance mode.
*
*     1.5    06/30/2015    EM       Removed Surveillance Analysis related code
*
*     1.6    06/30/2015    EM       Calculation of a baseline table (similar to table 1) for TYPE1 and TYPE2 requests.  
*
*     1.7    12/23/2015    EM       Changed age group names to be consitent with table 1.  
*
*     1.8    01/15/2016    VC       Allow for greater flexibility specifying the covariate assessment window.  
*									Create combined covariate variables.
*
*	  1.9    04/21/2016	   DM  		Added a warning when no baseline table is produced because no 
*									users were identified in the request
*
*	  1.10   08/10/2016	  AP(SOC)	Added 2 new HDPS dimension files (DX10 and PX10) for ICD10 (QCI-186)
*
*	  1.11   08/15/2016    DM       Added Surveillance Analysis related code removed in 1.5
*
*	  1.12   10/17/2016    DM       Fixed a couple of issues regarding Surveillance Analysis
*
*	  1.13   11/30/2016	  AP		Removed infolder libname from Monitoring File sort to accomodate 
*									truncation by DPMax Date	
*
*	  1.14   12/09/2016	  AP		Remove KeepAllDum as a parameter in the ComorbFile
*
***************************************************************************************************;


%macro ms_cidacov();

%put =====> MACRO CALLED: ms_cidacov v1.14;

%ms_starttimer(START=CIDACOV);

    %put Macro will further read raw data using the prefix Libname: &DataLib.;

/******************/
/* Pre-Processing */
/******************/

*List of patients from CIDANum for type=2;

    proc sort nodupkey data=DPLocal.&RUNID._mstr out=PtsList(keep=Patid);
    by Patid;
    where type<=2;
    run;
    proc sort nodupkey data=DPLocal.&RUNID._mstr out=&RUNID._mstr;
    by Group Patid IndexDt;
    where type<=2;
    run;

*Creating the number of type 2 groups to create cohorts;

    %isdata(dataset=infolder.&Type1File.);
    %if %eval(&NOBS. > 0) %then %do;
      data &Type1File.;
      set infolder.&Type1File.;
      Blackoutper=.; 
      run;
    %end;
    %isdata(dataset=infolder.&Type2File.);
    %if %eval(&NOBS. > 0) %then %do;
      data &Type2File.;
      set infolder.&Type2File.;
      run;
    %end;


    data _Type2COHORTFILE;
    set infolder.&COHORTFILE.;
    where upcase(type2)="Y" or upcase(type1)="Y";
    if upcase(type1)="Y" then call symput("Type2File","&Type1File.");
    run;
    %isdata(dataset=_Type2COHORTFILE);
    data _null_;
    call symputx("Ngroup","&nobs.");
    run;

*restrict the next steps to type 2 analyses only 
 (must have at least one patient);
%isdata(dataset=&RUNID._mstr);
%if %eval(&NOBS. > 0) %then %do;

%global NumCovars;
%let NumCovars=0;
%global NumComb;
%let NumComb=0;

%ISDATA(dataset=infolder.&COVARIATECODES.);	    
%IF %EVAL(&NOBS.>0) %THEN %DO;					
	proc sql noprint;	
	    select count (distinct CovarNum) 
	    into :NumCovars
	    from infolder.&COVARIATECODES.
		where codecat not in ("CC");
	quit;
	proc sql noprint;	
	    select count (distinct CovarNum) 
	    into :NumComb
	    from infolder.&COVARIATECODES.
		where codecat in ("CC");
	quit;
	%put Number of covars: &NumCovars;
	%put Number of covarComb: &NumComb;
%END;

*Assign pointer to prior DPLocal when running surveillance mode;
%global DPLOCALPOINTER;
%let DPLOCALPOINTER=DPLocal;
%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %let DPLocalPointer=DPLPrior;
%put NOTE: DPLOCAL for prior run is set to &DPLocalPointer.;	

/*********************************/
/* Compute Utilization Variables */
/*********************************/
 
    %macro RunUtil;

    %ISDATA(dataset=infolder.&UTILFILE.);	
    %IF %EVAL(&NOBS.>0) %THEN %DO;

        *Extracting Encounter Data;
        data _BaseEnc(keep=PatId ADate EncType);
        if 0 then set PtsList;
        declare hash ht (hashexp:16, dataset:"PtsList");
        ht.definekey('PatId');
        ht.definedone();

        do until(eof1);
          set &datalib.&encTable. end=eof1;
          if ht.find()=0 then output;
        end;
        stop;
        run;

        %do Group=1 %to &Ngroup.;

            *Processing Util parameters;
            Data _ITUtil;
            set infolder.&UTILFILE.;
            if _n_ = &Group.;
            call symputx('ITGROUP',strip(Group));

            if INCLINDEX=. then INCLINDEX=0; *defensive coding -  must be populated; 

            *always looking backwards (<0);
            call symputx("MEDUTILFROM",-1*MEDUTILFROM);
            *always looking forward (>=0);
        	if MEDUTILTO >=0 then call symputx("MEDUTILTO",MEDUTILTO);
            else call symputx("MEDUTILTO",0);

			/*
            *always looking backwards (negative);
        	call symputx("DRUGUTILFROM",-1*DRUGUTILFROM);
            *always looking forward (>=0);
        	if DRUGUTILTO >=0  then call symputx("DRUGUTILTO",DRUGUTILTO);
            else call symputx("DRUGUTILTO",0);
			*/

        	call symputx("CARESETTINGS",compress(CARESETTINGS,"',"));  *defensive coding; 
            if "&Analysis."="ps" or "&Analysis."="ads" then do;
                call symputx("CARESETTINGS","'AV' 'OA' 'IP' 'IS' 'ED'");  *defensive coding; 
                call symputx("BYENC","Y");
            end;
            else call symputx("BYENC","N");
        	call symputx("CSSTRAT",CSSTRAT);
            run;

            %put &CARESETTINGS.;

            data _null_;
            set infolder.&COHORTFILE.;
            if Group = "&ITGROUP." then do;
                call symputx('ITGROUP',strip(Group));
                if upcase(type2)="Y" then call symputx('type',2);
                else call symputx('type',1);
            end;
            run;

            %put &group. &ITGROUP.;

            *Creating index reference file;
            Data _IndexFile;
            set &RUNID._mstr;
            if Group = "&ITGROUP.";
            Keep Patid Group IndexDt;
            run;

            *Calculate Utilization;
            %MS_UTIL(ENCOUNTERFILE=_BaseEnc,		
              	     INDEXFILE=_IndexFile,
            	     INDEXDT=IndexDt, 
            	     PRIORDAYS=&MEDUTILFROM.,
            	     INCLINDEX=&MEDUTILTO.,
            	     CARESETTINGS=&CARESETTINGS., 
            	     BYENC=&BYENC.,
            	     CSSTRAT=&CSSTRAT.,
            	     OUTFILE=_ITUtil 
                 );  

            *Post Process;
            data _ITUtil;
            set _ITUtil;
            format group $30.;
            Group="&ITGROUP."; 
            length NumVisits $10.;
            format NumVisits $10.;
            NumVisits=NumVisit;
            drop NumVisit;
            run;

            *Combine Util for multiple groups;
            %IF %EVAL(&group.=1) %THEN %DO;
                data _AllUtil;
                set _ITUtil;
                run;
            %END;
            %ELSE %DO;
                proc append base=_AllUtil
                            data=_ITUtil force;
                run;
            %END;

            *Clean-up;
            proc datasets library=work nolist nowarn;
            delete _ITUtil _IndexFile;
            quit;

	        proc sort data=_AllUtil;
	        by Group PatId IndexDt;
	        run;
     	%END; *do loop;

		%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
		    data _NewPatients;
		    set DPLocal.&RUNID._mstr;
		    where &PERIODIDSTART. <= IndexLook  <= &PERIODIDEND.;
		    keep Group PatId IndexDt;
		    run;

		    proc sort nodupkey;
		    by Group PatId IndexDt;
		    run;

		    data _AllUtil;
		    merge _AllUtil(in=a)
		          _NewPatients(in=b);
		    by Group PatId IndexDt;
		    if a and b;
		    run;
		%end;

        data DPLocal.&RUNID._mstr;
        merge DPLocal.&RUNID._mstr
              _AllUtil;
        by Group PatId IndexDt;
        run;

        %ISDATA(dataset=infolder.&DRUGCLASSFILE.);	
        %IF %EVAL(&NOBS.>0) %THEN %DO;

            data _BaseTmp;
    		set dplocal.&RUNID._mstr;
    		where type<=2;
    		keep patid group indexdt;
    		run;

    	    *Adding DRUGUTIL window to base file;
    	    data _BaseTmp;
    	    if 0 then set infolder.&UTILFILE.(Keep=Group DrugUtilFrom DrugUtilTo);
    	    declare hash ht (hashexp:16, dataset:"infolder.&UTILFILE.");
    	    ht.definekey('Group');
    	    ht.definedata('DrugUtilFrom','DrugUtilTo'); 

    	    ht.definedone();

    	    do until(eof1);
    	    set _BaseTmp end=eof1;
        	    if ht.find()=0 then do;
                if .<DrugUtilFrom < 0 then DrugUtilFrom=-1*DrugUtilFrom;
                *if .<DrugUtilTo < 0 then DrugUtilTo=.;
                output;
                end;
    	    end;
    	    stop;
    	    run;

            *calculate NumGeneric-NumClass (v12);
    %put &datalib. &distable.;
            *extract Rx data in covariatewindow before IndexDt;
            proc sql noprint;
            create table _UtilRx as
            select Pts.Group, 
                   Pts.PatId,
                   Pts.IndexDt,
                   disp.NDC
            from _BaseTmp as pts,
                 &DataLib.&DisTable. as disp
            where pts.PatId = disp.Patid and 
                  %MS_PeriodsOverlap(period1=pts.IndexDt-COALESCE(pts.DrugUtilFrom,99999) COALESCE(pts.indexdt+DrugUtilTo/*-1*/,99999),                             
                                     period2=disp.RxDate);
            quit;

            *get Generic ClassName;
            proc sql noprint;
            create table _UtilRxClassGen as
            select Pts.Group, 
                   Pts.PatId,
                   Pts.IndexDt,
                   RxCl.Generic,
                   RxCl.ClassName,
                   1 as one
            from _UtilRx as pts,
                 infolder.&DRUGCLASSFILE. as RxCl
            where pts.NDC = RxCl.ndc;
            quit;

            *Count number of rows in dispensing table with NDC in drugclass file ;
            proc sql noprint;
            create table _NumRx as
            select Pts.Group, 
                   Pts.PatId,
                   Pts.IndexDt,
                   1 as one
            from _UtilRx as pts,
                 (select distinct ndc from infolder.&DRUGCLASSFILE.) as RxCl
            where pts.NDC = RxCl.ndc;
            quit;
            
            proc means data=_NumRx noprint nway;
            var one;
            class Group PatId IndexDt;
            output out=_NumRx(drop=_:) sum=NumRx;
            run;

            *Count number of disctinct Generic;
            proc sort nodupkey data=_UtilRxClassGen out=_Generic;
            by Group PatId IndexDt Generic;
            run;
            proc means data=_Generic noprint;
            var one;
            by Group PatId IndexDt;
            output out=_Generic(drop=_:) sum=NumGeneric;
            run;

            *Count number of disctinct ClassName;
            proc sort nodupkey data=_UtilRxClassGen out=_ClassName;
            by Group PatId IndexDt ClassName;
            run;
            proc means data=_ClassName noprint;
            var one;
            by Group PatId IndexDt;
            output out=_ClassName(drop=_:) sum=NumClass;
            run;

            *combine all three metrics;
            data _combine;
            merge _Generic
                  _ClassName
                  _NumRx;
            by Group PatId IndexDt;
            run;

			%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
			    data _combine;
			    merge _combine(in=a)
			          _NewPatients(in=b);
			    by Group PatId IndexDt;
			    if a and b;
			    run;

			    %if %eval(&PERIODIDSTART.>1) %then %do;			      	  

			      %macro PriorPeriods;
			        *complete covariates with pre-indexdt values from prior periods;
			        data _PriorPeriods;
			        set &DPLOCALPOINTER..&RUNID._ads_psmatch_%eval(&PERIODIDSTART.-1);
			        by Group PatId IndexDt;
			        keep Group PatId IndexDt NumGeneric NumClass NumRx;
			        run;
			      %mend;
			      %PriorPeriods;

			      data _baseTmp;
			      merge _baseTmp
			            _PriorPeriods;
			      by Group PatId IndexDt;
			      run;

			      proc datasets library=work nolist nowarn;
			      delete _PriorPeriods;
			      run;
			    %end;

			%end;

            *put Back in _Base;        
            data _baseTmp;
            merge _baseTmp(in=a)
                  _combine(keep=Group PatId IndexDt NumRx NumGeneric NumClass); *combine only contains new patients - other vars already updated;
            by Group PatId IndexDt;
            if a;  
            run;

            *Update Master file with utilization data;
            data DPLocal.&RUNID._mstr;
            merge DPLocal.&RUNID._mstr
                  _basetmp;
            by Group PatId IndexDt;
			if missing(NumRx) then NumRx=0;
			if missing(NumGeneric) then NumGeneric=0;
			if missing(NumClass) then NumClass=0;
            run;

            proc datasets library=work nolist nowarn;
            delete _BaseEnc IndexFile _:;
            quit;
        %END;
        %ISDATA(dataset=infolder.&DRUGCLASSFILE.);	
        %IF %EVAL(&NOBS.=0) %THEN %DO;
    		data DPLocal.&RUNID._mstr;
    		set DPLocal.&RUNID._mstr;
    		NumRx=.;
    		NumGeneric=.;
    		NumClass=.;
    		run;
        %END;
    %END;  
	%ELSE %DO;		
		data DPLocal.&RUNID._mstr;
		set DPLocal.&RUNID._mstr;
		ExactNumVisit=.;
		NumVisits_AV=.;
		NumVisits_IP=.;
		NumVisits_ED=.;
		NumVisits_OA=.;
		NumVisits_IS=.;
		NumVisits="";
		NumRx=.;
		NumGeneric=.;
		NumClass=.;
		run;
	%END;	*util calcs;
    %mend;


    %RunUtil;

/*********************************************/
/* and pre-extract the data for ADS          */ 
/* and Charlson Comorbidity/Elixhauser Index */
/*********************************************/

    %macro RunCovExtraction;

     *Process codes for extraction of records for Comorbs;
    %ISDATA(dataset=infolder.&Comorbfile.);	
    %let ComborbsCodes=;
    %IF %EVAL(&NOBS.>0) %THEN %DO;
        *Adapt for different lengths to avoid w a r n i n g s later;
        data _ComborbsCodes;
        length codetype $4. code $11. CODECAT $2.;
        format code $11.;
        set infolder.Comorbcodes;
        if code_grp="DIAG" then CODECAT="DX";
        Comorbcode=1;
        output;
        if wildcard="Y" then do;
            if Length(code)=3 then do;
                code=strip(code)||"*";
                output;
            end;
            if Length(code)<=4 then do;
                code=strip(code)||"*";
                output;
            end;
        end;
        run;
        %let ComborbsCodes=_ComborbsCodes(keep=Code CodeCat Codetype Comorbcode );
    %END;

     *Process codes for extraction of records for Covars;


    %ISDATA(dataset=infolder.&COVARIATECODES.);	
    %let CovarCodes=;
    %let CovarCodes2=;
    %IF %EVAL(&NOBS.>0) %THEN %DO;

	   	*Select covariates codes (not combination of covatiates);
		data _&COVARIATECODES.;
		set infolder.&COVARIATECODES.;
		where codecat not in ("CC");
		run;

        data _CovarCodes;
        set _&Covariatecodes.;
        Comorbcode=0;
        run;	
	%END;
	%ELSE %DO;
		data _CovarCodes;
		studyname = "";
        Code = "";
		CodeCat = "";
		Codetype = "";
		CARESETTINGPRINCIPAL = "";
		CovarNum = 0; 
		Comorbcode = 0;
		CovFrom=.;
		CovTo=.;
        run;
	%END;
        %let CovarCodes=_CovarCodes(keep=Code CodeCat Codetype CARESETTINGPRINCIPAL studyname CovarNum Comorbcode covfrom covto);
        %let CovarCodes2=if CodeCat in("PX","DX") and COvarnum >0  then output CovMeds;        

        proc sort nodupkey data=_CovarCodes out=studynames(keep=studyname CovarNum);
        by CovarNum;
        run;

    *Test whether extraction is necessary;
    %IF %LENGTH(&CovarCodes.) | %LENGTH(&Comorbfile.) %THEN %DO;
        * Extraction needed: Create extraction lookup files;    
        data _CovPx
             _CovDx
             _CovRx
             CovMeds;
        set &ComborbsCodes.
            &CovarCodes.;
        if CodeCat="PX" then output _CovPx;
        if CodeCat="DX" then output _CovDx;
        if CodeCat="RX" then output _CovRx;
        *if CodeCat in("PX","DX") and COvarnum >0  then output CovMeds;
        &CovarCodes2.;
        run;


    %MS_ProcessWildcards(InFile=_CovDx, 
                         CodeVar=code, 
                         OutFile=_CovDx);

    %ms_extractmeds(datafile=&DataLib.&proctable.,
                   datacode=PX ,
                   datacodetype=PX_codetype ,
                   lookfile=_CovPx, 
                   outfile=_procextract(drop=EncounterID Provider)
                   );

    %ms_extractmeds(datafile=&DataLib.&diatable.,
                   datacode=DX,
                   datacodetype=DX_codetype,
                   lookfile=_CovDx, 
                   outfile=_diagextract(drop=EncounterID Provider),
                   ExtraVars=Comorbcode
                   );

    *Combine _diags and _procs for envelope later;
    proc sql noprint;
    create table PreMeds as 
    select proc.PatId,
           proc.Adate,
           compress(proc.PX,' .')as Code length 11,
           "  " as Pdx,
           proc.EncType,
           PX_Codetype as Codetype,
           "PX" as CodeCat,
           1 as RxSup length 4,
           1 as RxAmt length 4,
           proc.Comorbcode,
		   proc.Covfrom,
		   proc.CovTo
    from _procextract as proc
    union corr all
    select diag.PatId,
           diag.Adate,
           compress(diag.DX,' .') as Code length 11,
           diag.EncType,
           diag.Pdx,
           diag.DX_Codetype as Codetype,
           "DX" as CodeCat,
           1 as RxSup length 4,
           1 as RxAmt length 4,
           diag.Comorbcode,
		   diag.Covfrom,
		   diag.CovTo
    from _diagextract as diag;
    quit; 

	data PreMeds(rename=DummyDate=ADate);
	format DummyDate mmddyy10.;
	length DummyDate 4.;
	set PreMeds;
	DummyDate=ADate;
	drop ADate;
	run;

    %MS_ENVELOPE(INFILE=PreMeds,		                /*Name of SAS dataset with diagnosis and/or procedure claims data*/
                 ENCOUNTERINFILE=&DataLib.&ENCTABLE.,	/*Name of SAS dataset with encounter data*/
                 OUTFILE=PreMeds);      	            /*Name of output file with reclassified claims*/ 

    *Codes must be unique to avoid claim duplication.;
/*    proc sort nodupkey data=_CovRx;*/
/*    by code;*/
/*    run;*/

    %ms_extractdrugs(datafile=&DataLib.&distable.,
                     datavar=NDC ,
                     lookfile=_CovRx(drop=codecat CodeType), 
                     lookvar=code , 
                     outfile=Drugs);
    %put &DataLib.;
    %IF %str("&DataLib.")=%str("indata.") %THEN %DO;
        %ms_delpatients(datafile=PreMeds,
                       ptsfile=&PTSTOEXCLUDE.,
                       Outfile=PreMeds);        
        %ms_delpatients(datafile=Drugs,
                       ptsfile=&PTSTOEXCLUDE.,
                       Outfile=Drugs);
    %END;

data ElixMeds (drop=covfrom covto)
     PreMeds;
set PreMeds;
if Comorbcode eq 1 then output ElixMeds;
else output PreMeds;
run;
%END;
    proc datasets library=work nolist nowarn;
    delete _:;
    quit;
    %mend;
    %RunCovExtraction;

/******************************************/
/* Charlson Comorbidity/Elixhauser Index) */
/******************************************/

    %macro RunCciElix;

    %ISDATA(dataset=infolder.&Comorbfile.);	
    %IF %EVAL(&NOBS.>0) %THEN %DO;

        *Break down Meds in procs and diags;
            data _cci_elix_dx
                 _cci_elix_px;
            set ElixMeds;
            *where Comorbcode eq 1;
            if Codecat="DX" then output _cci_elix_dx;
            if Codecat="PX" then output _cci_elix_px;
            run;
        %put &Ngroup.;
        %do Group=1 %to &Ngroup.;

            *Process input file;
            data _null_;
               set infolder.&Comorbfile.;
               if _n_ = &Group. then do;
                    call symputx('ITGROUP',Group);
                    call symputx('priordays',/*priordays*/-1*comorbFrom);  *put pback positive for tools;
                    call symputx('indexdays',indexdays);
                    call symputx('ccigroup',ccigroup);
                   /* call symputx('keepalldum',keepalldum);*/
                    call symputx('inclindex',/*inclindex*/max(0,comorbTo));
                end;
            run;

            %put &ITGROUP. &priordays. &indexdays. &ccigroup. /*&keepalldum.*/ &inclindex.;

            *Update master list with 

            *this has been check in CIDANum;
            *Note 3: if a member does not have PRIORDAYS +INDEXDAYS days of 
                     continuous enrollment before the treatment episode, 
                     the treatment episode is excluded. 

            *The Charlson/Elixhauser combined comorbidity score will be calculated;


            data _IndexFile;
            set &RUNID._mstr;
            where Group="&ITGROUP.";
            Keep Patid indexdt;
            run;

            %ms_cci_elix(PRIORDAYS=&priordays.,
                         INDEXDAYS=&indexdays.,
                         INCLINDEX=&inclindex., 
                         INFILE=_IndexFile, 
                         PATID=PatId,
                         INDEXDT=IndexDt, 
                         PREOUTFILE=_PreOutFile,
                         DIAGFILE=_cci_elix_dx,
                         DX_DT_VAR=Adate,
                         DX_CODETYP=Codetype,
                         DX_CODE=Code,
                         PROCFILE=_cci_elix_procs, 
                         PX_DT_VAR=Adate,
                         PX_CODETYP=Codetype,
                         PX_CODE=Code,
                         CCI_LOOKUP=infolder.Comorbcodes, 
                         OUTFILE=_ITcci_elix,
                         CCIGROUP=&ccigroup.,
                         KEEPALLDUM=0);   /*&keepalldum.*/


        *Post Process;
            data _ITcci_elix;
            set _ITcci_elix;
            format group $30.;
            Group="&ITGROUP."; 
            length ccielixgrp $10.;
            format ccielixgrp $10.;
            ccielixgrp=PCOMBINED_SCORE;
            run;

            *Combine Util for multiple groups;
            %IF %EVAL(&group.=1) %THEN %DO;
                data _ALLcci_elix;
                set _ITcci_elix;
                run;
            %END;
            %ELSE %DO;
                proc append base=_ALLcci_elix
                            data=_ITcci_elix force;
                run;
            %END;

            *Clean-up;
            proc datasets library=work nolist nowarn;
            delete _ITcci_elix _IndexFile _PreOutFile;
            quit;

        %END; *do loop;

        proc sort data=_ALLcci_elix;
        by Group PatId IndexDt;
        run;

		%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;

		    data _NewPatients;
		    set DPLocal.&RUNID._mstr;
		    where &PERIODIDSTART. <= IndexLook  <= &PERIODIDEND.;
		    keep Group PatId IndexDt;
		    run;

		    proc sort nodupkey;
		    by Group PatId IndexDt;
		    run;

		    data _ALLcci_elix;
		    merge _ALLcci_elix(in=a)
		          _NewPatients(in=b);
		    by Group PatId IndexDt;
		    if a and b;
		    run;
		%end;

        *Update Master file with utilization data;
        data DPLocal.&RUNID._mstr;
        merge DPLocal.&RUNID._mstr
              _ALLcci_elix;
        by Group PatId IndexDt;
        run;

        proc datasets library=work nolist nowarn;
        delete _BaseEnc IndexFile _: 
               Elix_infile_sort Dsout Cci_diag_temp Cci_diag_lookup;
        quit;

    %end;
	%else %do;		
		data DPLocal.&RUNID._mstr;
		set DPLocal.&RUNID._mstr;
		PCOMBINED_SCORE="";
		ICOMBINED_SCORE="";
		PCOMBINED_SCORE_NUM=.;
		ICOMBINED_SCORE_NUM=.;
		ccielixgrp="";
		run;
	%end;
    %mend;

    %RunCciElix;

/***********************/
/* Prepare psmatch ADS */
/***********************/

    %macro psmatchADS;
    %put &analysis.;
    %if %str(%UPCASE("&ANALYSIS."))=%str("PS") or %str(%UPCASE("&ANALYSIS."))=%str("ADS") %then %do;

        *Creating the base file to extract data 
         and then divide into different PeriodId;
        data _Base;
        set DPLocal.&RUNID._mstr;
        where type<=2;

        rename AgeGroup=Age_Cat
               FEventDt=EventDt
               NumVisits_IP=NumIP
               NumVisits_ED=NumED
               NumVisits_AV=NumAV
               NumVisits_OA=NumOA
               NumVisits_IS=NumIS
               PCOMBINED_SCORE_NUM=COMORBIDSCORE;

        keep  PatID IndexDt Year Group Age AgeGroup Sex Race FEventDt tte
              PCOMBINED_SCORE_NUM NumVisits_IP NumVisits_ED NumVisits_AV NumVisits_OA NumVisits_IS
              EpisodeEndDt EpisodeType LastLookFollowed: NumGeneric NumClass NumRx;
        run;

        Proc sort data=_base;
        by Group Patid IndexDt;
        run;



        /**************************/
        /* Creating Covar1-CovarN */
        /**************************/

        %MS_ProcessWildcards(InFile=CovMeds, 
                             CodeVar=code, 
                             OutFile=CovMeds);

        *Break out CaresettingPrincipal into Caresetting and Principal;
        %ms_caresettingprincipal(InFile=CovMeds, 
                                 Var=CareSettingPrincipal, 
                                 OutFile=CovMeds);

        %let datafile=meds;
        %let lookfile=CovMeds; 
        %let outfile=_ITMeds;

        *Extracting meds using EncType and PDX to meds;
        %ms_loopmeds(datafile=PreMeds,
                     lookfile=CovMeds, 
                     outfile=_Meds);

        *Combine records;
        Data _CombineRecords;
        length Adate ExpireDt 4.;  * to avoid w a r n i n g s;
        set Drugs(rename=RxDate=ADate)
            _meds;

        format Adate ExpireDt date9.;
        ExpireDt=Adate+max(1,RxSup)-1;
        where studyname ne '';
        Keep PatId Adate ExpireDt studyname CovarNum covfrom covto;
        run; 

        proc datasets library=work nowarn nolist;
        delete _Drugs _meds CovMeds PreMeds;
        quit;

        /*Finding those claims in covariatewindow*/
        proc sql noprint;
        create table _Covar as
        select distinct pts.Patid,
                        pts.Group,
                        pts.IndexDt,
                        Rec.studyname,
						Rec.covfrom,
						Rec.covto,
                        Rec.CovarNum
        from _base as pts, 
             _CombineRecords as Rec
        where pts.Patid = rec.Patid and 
             %MS_PeriodsOverlap(period1=coalesce(pts.IndexDt+covFrom,-99999) COALESCE(pts.indexdt+CovTo/*-1*/,99999),                                             
                                period2=Rec.ADate )
        order by pts.Group,
                 pts.Patid,
                 pts.IndexDt;
        quit;

        *Squaring;
        data _Covar;
        set  studynames
             _Covar;
        Rec=1;
        run;

        *One record per Patid Group IndexDt;
        proc transpose data=_Covar out=_Covar(drop=_NAME_) prefix=COVAR;
        var Rec;
        by Group Patid IndexDt;
        id Covarnum;
        idlabel studyname;
        run;

		/******************************/
		/*Verify if any combcovariates*/
		/******************************/		
	    %ISDATA(dataset=infolder.&COVARIATECODES.);	
		%IF %EVAL(&NOBS.>0) %THEN %DO;
			data _COMBO;
			set infolder.&COVARIATECODES.;
			where codecat in ("CC");
			run;

			%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
			    data _NewPatients;
			    set DPLocal.&RUNID._mstr;
			    where &PERIODIDSTART. <= IndexLook  <= &PERIODIDEND.;
			    keep Group PatId IndexDt;
			    run;

			    proc sort nodupkey;
			    by Group PatId IndexDt;
			    run;
			%end;

			%ISDATA(dataset=_COMBO);
			%IF %EVAL(&NOBS.>=1) %THEN %DO;				
				data _COMBO;
				set _COMBO;
				format RuleLong $1000.;
				RuleLong=code;
				%do j=1 %to &numcovars.;
					RuleLong=left(TRANWRD(cat(' ',rulelong,' '),(" &j. ")," COVAR&j. "));
					RuleLong=left(TRANWRD(cat(' ',rulelong,' '),("(&j. "),"(COVAR&j. "));
					RuleLong=left(TRANWRD(cat(' ',rulelong,' '),(" &j.)")," COVAR&j.)"));
				%end;
				run;

				*Create CombCovar variables;
				%do i=(&numcovars.+1) %to (&numcovars.+&numcomb.);
					data _null_;
					set _COMBO (where=(Covarnum=&i.));
					call symput ('Rule',RuleLong);
					call symput ('Label',Studyname);
					run;
					%put &Rule.;

					data _covar;
					set _covar;
					COVAR&i.=0;
					if &rule. then do;
					COVAR&i.=1;
					end;
					label COVAR&i.="&Label.";
					run;

				%end;*do loop numcomb;

				%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;
				    
					proc sort data=_Covar;
				    by Group PatId IndexDt;
				    run;

				    data _Covar;
				    merge _Covar(in=a)
				          _NewPatients(in=b);
				    by Group PatId IndexDt;
				    if a and b;
				    run;

				%end;

		        *Add dummies to master file (and will also order the vars by covarN);
		        %macro PutBack;
		        data _base;
		        merge _base(in=a)
		              _Covar(keep=Group PatId IndexDt  %do j=1 %to &numcovars.;%str(covar&j. )%end;
													   %do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k. )%end;);
		        by Group PatId IndexDt;
		        if a;  
		        run;
		        %mend;
		        %PutBack;
			%END;
			%ELSE %DO;
				%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;				    

				    data _Covar;
				    merge _Covar(in=a)
				          _NewPatients(in=b);
				    by Group PatId IndexDt;
				    if a and b;
				    run;

				%end;

		        *Add dummies to master file (and will also order the vars by covarN);
		        %macro PutBack;
		        data _base;
		        merge _base(in=a)
		              _Covar(keep=Group PatId IndexDt  %do j=1 %to &numcovars.;%str(covar&j. )%end;);
		        by Group PatId IndexDt;
		        if a;  
		        run;
		        %mend;
		        %PutBack;
			%END;
		%END;

        *Creating individual PeriodId files;
        
        proc sort data=&Monitoringfile. out=_Looks;
        by PeriodId;
        *where &PERIODIDSTART. <= PeriodId  <= &PERIODIDEND.;
        run;

        data _Looks;
        set _looks end=eof;
        lastEndDt=lag(enddate);
        format incrstartfollowup date9.;
        incrstartfollowup=startfollowup;
        if _N_>1 then incrstartfollowup=lastEndDt+1;        
        drop lastEndDt;
        run;

        proc sql noprint undo_policy=none;;
        create table _base as
        select distinct base.*,
                        look.PERIODID as time
        from _Base as base, 
             _Looks as look
        where incrstartfollowup <= IndexDt <= enddate;
        quit;

        %macro forAdsloop;
        %do i=&PERIODIDSTART. %to &PERIODIDEND.;

            %if %eval(&i.=&PERIODIDSTART.) %then %do;
              proc sql noprint undo_policy=none;
              create table _base as
              select b.*,
                     l.Blackoutper
              from _base as b left join
                   &TYPE2FILE. as l 
                   on b.Group=l.Group
              order group, patid, indexdt;
              quit;
            %end;
 
            data DPLocal.&RUNID._ads_psmatch_&i.;
            set _Base;
      		if _N_=1 then set _Looks(keep=PERIODID enddate where=(PERIODID=&i.));

            if missing(NumGeneric) then NumGeneric=0;
            if missing(NumClass) then NumClass=0;
            if missing(NumRx) then NumRx=0;

            if Blackoutper=. then Blackoutper=0;

            *Adjust tte for episodes going beyond the PERIODID;
            if IndexDt <= enddate and EpisodeEndDt > min(LastLookFollowedDt,enddate) then do;
               if EventDt > min(LastLookFollowedDt,enddate) then EventDt = .; 
               *tte = min(tte,enddate - IndexDt +1 -  BlackOutPer);
               tte = min(LastLookFollowedDt,EventDt,enddate) - IndexDt +1 -  BlackOutPer;
            end;

            *Variables needded for the ADS;
            Followuptime_ITT=.;
            Event_ITT=0;
            if EpisodeType="ITT" then do;
                Followuptime_ITT=tte;
                Event_ITT=EventDt>0;
            end;


            Followuptime_astreated=.;
            Event_astreated=0;

            if EpisodeType="EPI" then do;
                Followuptime_astreated=tte;
                Event_astreated=EventDt>0;				
            end;

            *fill covars with 0s;
            %do j=1 %to &numcovars.;%str(covar&j.=covar&j.>0;)%end;
			%do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k.=covar&k.>0; )%end;

            *Output incremental datasets;
            if IndexDt <= enddate then output DPLocal.&RUNID._ads_psmatch_&i.;

            *Cleanup to keep only required in specs;
            drop enddate EpisodeEndDt EpisodeType PERIODID tte BlackOutPer;
            run;

			*Carry forward information from previous look in ads_psmatch;
			%IF %EVAL(&i.>1) %THEN %DO;
				%let previous = %eval(&i-1); 
				data DPLocal.&RUNID._ads_psmatch_&i.;
				merge DPLocal.&RUNID._ads_psmatch_&i.
              	  	  &DPLOCALPOINTER..&RUNID._ads_psmatch_&previous. (where=(Time < &PERIODIDSTART. and LastLookFollowed > 0)
															  keep=Time LastLookFollowed Group PatId IndexDt Followuptime_: Event_: 
															  %do j=1 %to &numcovars.;%str(covar&j. )%end;
															  %do k=(&numcovars.+1) %to (&numcovars.+&numcomb.);%str(covar&k. )%end;);
	        	by Group PatId IndexDt;
	        	run;
			%END;
 
*Create Baseline table;

*Strip age_cat of special characters;
  data _RawData;
  set  DPLocal.&RUNID._ads_psmatch_&i.; 
  by group patid indexdt;
  Age_cat=TRANSLATE(Age_cat,"_","-"); 
  Age_cat=upcase(compress(Age_cat,' <>+-'));
  patient=1;
  run;

*Create age group dummies - can be multiple index per patient;
  proc transpose data=_RawData out=_AgeGroups(keep=group patid indexdt Age:)  prefix=AGE;
  var patient;
  by group patid indexdt;
  id Age_cat;
  run;

*Create sex dummies - can be multiple index per patient;
  proc transpose data=_RawData out=_Genders(keep=group patid indexdt Sex_:)  prefix=Sex_;
  var patient;
  by group patid indexdt;
  id Sex;
  run;

*Merge back agegroup and sex dummies;
  data _RawData;
  merge _RawData(drop=patient)
        _AgeGroups
		_Genders;
  by group patid indexdt;

  patient=0;
  *create a new patient indicator to count distinct patients;
  if first.patid then patient=1;

  if not first.patid then do;
	array Sexes Sex_:;
	do over Sexes;
	  Sexes=.;
	end;
  end;

  run;

*Process discrete vars;
  proc means data=_RawData(drop=age age_cat) nway noprint;
  var patient Age: Sex_: 
		%if %eval(&numcovars. ge 1) %then %do; covar: %end;
		%if %eval(&numcomb. ge 1) %then %do; covar: %end;
  	  Event_ITT Event_astreated;
  class group;
  output out=_Discrete(drop=_:) sum=;
  run;

*Process continuous vars;
  proc means data=_RawData nway noprint;
  var Followuptime_ITT Followuptime_astreated 
      Age COMORBIDSCORE 
      NumAV NumOA NumIP NumIS NumED 
      NumGeneric NumClass NumRx;
  class group;
  output out=_Continuous(rename=_freq_=N_episodes drop=_type_)
        mean=mean_Followuptime_ITT mean_Followuptime_astreated 
             mean_Age mean_COMORBIDSCORE 
             mean_NumAV mean_NumOA mean_NumIP mean_NumIS mean_NumED 
             mean_NumGeneric mean_NumClass mean_NumRx
         std=std_Followuptime_ITT std_Followuptime_astreated
             std_Age std_COMORBIDSCORE 
             std_NumAV std_NumOA std_NumIP std_NumIS std_NumED 
             std_NumGeneric std_NumClass std_NumRx;
  run;

*merge the two;
  data msoc.&RUNID._baseline_&i.;
  merge _discrete
        _continuous;
  by group;
  run;

*add missing groups if any; 
proc sql noprint;
	create table _MissingGroups as
	select distinct (Group), 0 as patient, 0 as N_Episodes  
	from infolder.&COHORTFILE.
	where group not in
	  (select distinct (Group) 
	   from msoc.&RUNID._baseline_&i.
	  )
	;
quit;
	 
data msoc.&RUNID._baseline_&i.;
set msoc.&RUNID._baseline_&i.
	_MissingGroups;
run;

        %end;
        %mend forAdsloop;
        %forAdsloop;


        /*****************************/
        /* Claims_icddx.sas7bdat     */
        /* Claims_icdpx.sas7bdat     */
        /* Claims_cpt.sas7bdat       */
        /* Claims_hcpcs.sas7bdat     */
        /* Claims_drugclass.sas7bdat */
        /*****************************/
        %macro ClaimsFiles;
            %let HDPS=N;

            %ISDATA(dataset=HDPSSettings);
			*Note: HDPS settings contains groups where HDPS=Y;	
            %IF %EVAL(&NOBS.>0) %THEN %DO;
              data _ToSelect;
              set HDPSSettings;
              where upcase(HDPS)="Y";
              if upcase(HDPS)="Y" then call symputx("HDPS",upcase(HDPS));
              keep Group;
              run;
              %put &HDPS.;
            %END;

            %if %str(&HDPS.)=%str(Y) %then %do; 
            
               *Adding covariatewindow to mstr file and keep only patients in groups where HDPS is required;
                data _lookup(keep=Patid Group Indexdt);
                if 0 then set _ToSelect(Keep=Group);
                declare hash ht (hashexp:16, dataset:"_ToSelect");
                ht.definekey('Group');
                ht.definedone();

                do until(eof1);
                  set &RUNID._mstr end=eof1;
                  if ht.find()=0 then output;
                end;
                stop;
                run;

				*keep only new patients;
				%if "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %then %do;

				    proc sort nodupkey data=_NewPatients(keep=Group PatId);
				    by Group PatId;
				    run;

				    data _lookup;
				    merge _lookup(in=a)
				          _NewPatients(in=b);
				    by Group PatId;
				    if a and b;
				    run;
				%end;

                *Claims_icddx;
                proc sql noprint;
                create table &RUNID._Claims_icddx as
                select /*distinct*/ claims.Patid,
                                compress(claims.DX) as Code,
								claims.DX_Codetype,
                                claims.Adate,
                                pts.group
                from _lookup as pts, 
                     &DataLib.&Diatable. as claims
                where pts.Patid = claims.Patid and
                      %MS_PeriodsOverlap(period1=pts.IndexDt+coalesce(&MinHDPSWinfrom.,-99999) pts.IndexDt+coalesce(&MaxHDPSWinTo.,99999),                                             
                                         period2=claims.ADate)
                order by claims.Patid, 
                         pts.group;
                quit;

  				data DPLocal.&RUNID._Claims_icddx09
                     DPLocal.&RUNID._Claims_icddx10;
                set &RUNID._Claims_icddx; 

                if DX_Codetype in("09") then output DPLocal.&RUNID._Claims_icddx09;
				if DX_Codetype in("10","11") then output DPLocal.&RUNID._Claims_icddx10;
                drop DX_Codetype;
                run;

                *Claims_icdpx;
                proc sql noprint;
                create table &RUNID._Claims_icdpx as
                select /*distinct*/ claims.Patid,
                                compress(claims.PX) as Code,
                                pts.group,
                                claims.PX_Codetype,
                                claims.Adate
                from _lookup as pts, 
                     &DataLib.&Proctable. as claims
                where pts.Patid = claims.Patid and
                      %MS_PeriodsOverlap(period1=pts.IndexDt+coalesce(&MinHDPSWinfrom.,-99999) pts.IndexDt+coalesce(&MaxHDPSWinTo.,99999),                                             
                                         period2=claims.ADate)
                order by claims.Patid, 
                         pts.group;
                quit;

                data DPLocal.&RUNID._Claims_icdpx09
					DPLocal.&RUNID._Claims_icdpx10
                     DPLocal.&RUNID._Claims_cpt
                     DPLocal.&RUNID._Claims_hcpcs;
                set &RUNID._Claims_icdpx; 

                if PX_Codetype in("09") then output DPLocal.&RUNID._Claims_icdpx09;
                if PX_Codetype in("10","11") then output DPLocal.&RUNID._Claims_icdpx10;
                if PX_Codetype in("C2","C3","C4") then output DPLocal.&RUNID._Claims_cpt;
                if PX_Codetype in("HC","H3")      then output DPLocal.&RUNID._Claims_hcpcs;

                drop PX_Codetype;
                run;

                *Claims_drugclass;
                proc sql noprint;
                create table _Claims_drugclass as
                select /*distinct*/ claims.Patid,
                                claims.NDC as Code,
                                claims.Rxdate as Adate,
                                pts.group
                from _lookup as pts, 
                     &DataLib.&distable. as claims
                where pts.Patid = claims.Patid and
                      %MS_PeriodsOverlap(period1=pts.IndexDt+coalesce(&MinHDPSWinfrom.,-99999) pts.IndexDt+coalesce(&MaxHDPSWinTo.,99999),                                             
                                         period2=claims.RxDate) /*claims.ADate+RxSup-1*/;
                quit;

                *add Drugclass information;
                proc sql noprint;
                create table DPLocal.&RUNID._Claims_drugclass as
                select /*distinct*/ claims.Patid,
                                claims.group,
                                claims.adate,
                                ndc.classname as Code
                from infolder.&DRUGCLASSFILE. as ndc
                     inner join 
                     _Claims_drugclass as claims
                on ndc.ndc = claims.Code
                order by claims.Patid, 
                         claims.group;
                quit;

				%if ("&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p") & %eval(&PERIODIDSTART.>=2)  %then %do;

				  data DPLocal.&RUNID._Claims_icdpx09;
				  set  DPLocal.&RUNID._Claims_icdpx09
				      &DPLOCALPOINTER..&RUNID._Claims_icdpx09;
				  by Patid group;
				  run;

				  data DPLocal.&RUNID._Claims_icdpx10;
				  set  DPLocal.&RUNID._Claims_icdpx10
				      &DPLOCALPOINTER..&RUNID._Claims_icdpx10;
				  by Patid group;
				  run;

				  data DPLocal.&RUNID._Claims_cpt;
				  set  DPLocal.&RUNID._Claims_cpt
				      &DPLOCALPOINTER..&RUNID._Claims_cpt;
				  by Patid group;
				  run;

				  data DPLocal.&RUNID._Claims_hcpcs;
				  set  DPLocal.&RUNID._Claims_hcpcs
				      &DPLOCALPOINTER..&RUNID._Claims_hcpcs;
				  by Patid group;
				  run;

				  data DPLocal.&RUNID._Claims_icddx09;
				  set  DPLocal.&RUNID._Claims_icddx09
				      &DPLOCALPOINTER..&RUNID._Claims_icddx09;
				  by Patid group;
				  run;

				  data DPLocal.&RUNID._Claims_icddx10;
				  set  DPLocal.&RUNID._Claims_icddx10
				      &DPLOCALPOINTER..&RUNID._Claims_icddx10;
				  by Patid group;
				  run;

				  data DPLocal.&RUNID._Claims_drugclass;
				  set  DPLocal.&RUNID._Claims_drugclass
				      &DPLOCALPOINTER..&RUNID._Claims_drugclass;
				  by Patid group;
				  run;

				%end;

            %END;  *HDPS;
        %mend;
        %ClaimsFiles;

        proc datasets library=work nolist nowarn;
        delete Covmeds Drugs Monitoringfileflm Premeds Ptslist Studynames _:;
        quit;

    %END;
    %mend;
    %psmatchADS;
%end;
%else %do;
%put WARNING: No users were identified in this request. If a baseline table was specified it will not be produced.;
%end;

proc datasets library=work nowarn nolist;
delete PreMeds ElixMeds;
quit;

%ms_stoptimer(START=&CIDACOV.);


%put NOTE: ******** END OF MACRO: ms_cidacov v1.14 ********;

%mend ms_cidacov;
