****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidadenom.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 11/14/2016
* Version: 1.7
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  This macro will compute denominators.                                        
*   
*  Program inputs:                                                                                   
*  	-Many datasets created by ms_cidanum
* 
*  Program outputs:                                                                                                                                       
*  	-The DPLocal.&RUNID._DenomCounts output table
*  	-The DPLocal.&RUNID._&GEOG output tables
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.2      01/12/15     DM       Defensive coding added to eliminate warnings and errors triggeredby
*                                   mergenoby option values
*
*	 1.3	  01/22/15	   DM		Commented out the id parameter in the shaveoutside macro call
*
*	 1.4	  02/02/15	   SL(DM)	Corrected bug related to open ended condfrom/condto
*
*	 1.5	  01/15/16	   SL(VC)	Modify to include complex inclusion and Covariate enhancements
*
*    1.6      04/21/16	   DM  		Added HDPS window parameters in denominators algorithm
*
*	 1.7	  11/14/16     AP		Added denominator calculations for geographic location	
*
***************************************************************************************************;


%macro ms_cidadenom();

%put =====> MACRO CALLED: ms_cidadenom v1.7;

/*---------------------------------------------------*/
/* Original enrollment periods						 */
/*---------------------------------------------------*/
* Extract highest CONDFROM value among exclusions if any;
    %let MaxExclCondFrom=0;
    %macro wrapper;
    	%ISDATA(dataset=&INCLUSIONCODES.);
    	%IF %EVAL(&NOBS.>=1) %THEN %DO;
    		proc sql noprint;
    		select min(condfrom) into :MaxExclCondFrom 						
    		from &INCLUSIONCODES.(where=(group="&ITGROUP.")) 
    		where Condinclusion=0;
    		quit;
    	%END;
    	%put &MaxExclCondFrom.;
    %mend wrapper;
    %wrapper;


*If death censoring is required, it was already applied (Enr_End was previously truncated by CIDAnum);
    data _DenomOrig&Group. (keep=PatId EligEpisode Enr_Start Enr_End)
    	 _Denom&Group.;
    set Enr_&Group.;

    where Enr_End>=&startfollowup.;
    format  Enr_Start Enr_End DenomEnrStart DenomEnrEnd date9.;

    MaxLookBack=Max(0,&ENRDAYS.,&WASHPER.,&FUPWASHPER.,-1*&PRIORDAYS_BF.,&MINCOVFROM.,-&MaxExclCondFrom.,&L1HDPSFROM.);

    * Resctrict eligibility to Study period (minus maximum of required lookbacks);
    DenomEnrStart=Enr_Start;
    * Initially truncate start eligibility enrollment start by max of lookback periods;
    DenomEnrStart=DenomEnrStart + MaxLookBack;
    DenomEnrEnd=Enr_End;
    EligEpisode=episode;
    if DenomEnrEnd >= DenomEnrStart;
    drop DeathDt episode;

	*Create MaxLookBack macro variable;
	call symputx('MaxLookBack',MaxLookBack);
    run;
	%put &MaxLookBack.;

/*---------------------------------------------------*/
/* Inclusion required (POV3 Incl) for eligibility	 */
/*---------------------------------------------------*/  
proc datasets library=work nowarn nolist;
delete _denom1_stack:;
quit;

%macro wrapper;
	%ISDATA(dataset=&INCLUSIONCODES.);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;

		data _IT&INCLUSIONCODES.;
		set &INCLUSIONCODES.;
		where group="&ITGROUP.";
		run; 

		*For EACH condition, shave periods for all subconditions;
		%ISDATA(dataset=_IT&INCLUSIONCODES.);
		%IF %EVAL(&NOBS.>=1) %THEN %DO;
			%let MAXCOND=;
			proc sql;
			select max(cond) into: maxcond
			from _IT&INCLUSIONCODES.;
			quit;
			%put &MAXCOND.;

			%DO COND=1 %TO &MAXCOND.;
				*Determine how many levels of subconditions needed;
				%let MAXSUBCOND=;
				proc sql;
				select max(subcond) into: maxsubcond
				from _IT&INCLUSIONCODES. (where=(COND=&COND.));
				quit;
				*Determine if condition is an inclusion;
				proc sql;
				select max(CondInclusion) into: inclusion
				from _IT&INCLUSIONCODES. (where=(COND=&COND.));
				quit;
				%put &MAXSUBCOND. &INCLUSION.;

				*Determine if there is subexclusion in condition in order to calculate new maxlookback since one above is for CondInclusion=0;
				%let SUBEXCL=0;
	            data _null_;
	            set _IT&INCLUSIONCODES.(where=(COND=&COND.));
	            if subCondinclusion=0 then call symputx("SUBEXCL",1);
	            run;
	            %put SUBEXCL = &SUBEXCL.;

				*If there is subexclusion, determine new maxlookback for that specific condition before shaving;
				%IF &SUBEXCL.=1 %THEN %DO;
				    %let NewMaxExclCondFrom=0;
				    proc sql noprint;
				    select min(condfrom) into :NewMaxExclCondFrom 
				    from _IT&INCLUSIONCODES. 									
				    where COND=&COND. and SubCondInclusion=0;
				    quit;
				    %put &NewMaxExclCondFrom.;

				    data _Denom&Group._&cond. (rename=(newmaxlookback=MaxLookBack));
				    set _Denom&Group.;
					*If the subcond condfrom is greater in absolute value than the maxlookback value, then readjust denom period;
/*				    NewMaxLookBack=Max(-&MaxExclCondFrom.,-&NewMaxExclCondFrom.);*/
				    NewMaxLookBack=Max(0,&Maxlookback.,-&NewMaxExclCondFrom.);
				    * Resctrict eligibility to Study period (minus maximum of required lookbacks);
				    DenomEnrStart=Enr_Start;
				    * Initially truncate start eligibility enrollment start by max of lookback periods;
				    DenomEnrStart=DenomEnrStart + NewMaxLookBack;
				    DenomEnrEnd=Enr_End;
				    if DenomEnrEnd >= DenomEnrStart;
					  drop maxlookback;
				    run;
				%END;*If subexcl=1;
				%IF &SUBEXCL.^=1 %THEN %DO;
					data _Denom&Group._&cond.;
					set _Denom&Group.;
					run;
				%END;

				%DO SUBCOND=1 %TO &MAXSUBCOND.;
					*Determine if subcond is an inclusion or exclusion;
					%let SUBINCLUSION=0;
 					proc sql;
					select max(SubCondInclusion) into: subinclusion
					from _IT&INCLUSIONCODES. (where=(COND=&COND. and SUBCOND=&SUBCOND.));
					quit;
					%put &SUBINCLUSION.;

					*Merge time periods for each subcondition and shave for each;
					%IF &SUBINCLUSION. = 1 %THEN %DO;
						  data _EligIncl_&subcond.;
					    set _InclExcl;
					    where COND=&COND. and subcond=&SUBCOND.;
					    format EligStart EligEnd date9.;
					    EligStart=ADate-coalesce(CondTo,99999);
					    EligEnd=Expiredt-coalesce(CondFrom,-99999);
					    if EligStart<=EligEnd;
					    keep PatId ADate EligStart EligEnd;
					    run;

						%MergeTimePeriods(dataset=_EligIncl_&subcond.,
						                  DateStart=EligStart,
						                  DateEnd=EligEnd);	
		 
						%ms_shaveoutside(reffile=_EligIncl_&subcond.,
						              	 refstart=EligStart,
						               	 refend=EligEnd,  
									   	       KeepPartBf=Y,
						               	 ToShaveFile=_Denom&Group._&cond.,
						               	 ToShaveStart=DenomEnrStart,
						               	 ToShaveEnd=DenomEnrEnd,
						               	 outfile=_Denom&Group._&cond.);

						*Drop refstart and refend since _Denom&Group can be looped;
						data _Denom&Group._&cond.;
						set _Denom&Group._&cond.(drop=eligstart eligend);
						run;
					%END;*If subinclusion=1;
					%IF &SUBINCLUSION. = 0 %THEN %DO;
				        data _UneligExcl_&subcond.;
				        set _InclExcl;
					      where COND=&COND. and subcond=&SUBCOND.;
				        format UneligStart UneligEnd date9.;
				        UneligStart=ADate-coalesce(CondTo,99999);
				        UneligEnd=Expiredt-coalesce(CondFrom,-99999);
				        if UneligStart<=UneligEnd;
				        keep PatId ADate UneligStart UneligEnd;
				        run;

						%MergeTimePeriods(dataset=_UneligExcl_&subcond.,
						                  DateStart=UneligStart,
						                  DateEnd=UneligEnd);

						*Verify if there is data or else e.r.r.o.r when shaving;
						%ISDATA(dataset=_UneligExcl_&subcond.);
						%IF %EVAL(&NOBS.>=1) %THEN %DO;
						    %ms_shaveinside(reffile=_Denom&Group._&cond.,
						                   	refstart=DenomEnrStart,
						                    refend=DenomEnrEnd,  
										            id=EligEpisode, 				    
						                    ToShaveFile=_UneligExcl_&subcond.,
						                    ToShaveStart=UneligStart,
						                    ToShaveEnd=UneligEnd,
						                    outfile=_Denom&Group._&cond.);
						%END;*If there is data;
					%END;*If subinclusion=0;
				%END;*loop on subcond;
%put &INCLUSION.;
				%IF &INCLUSION.=1 %THEN %DO;
					proc append base=_Denom&Group._STACK_INC
						data=_Denom&Group._&cond.(keep=patid denomenrstart denomenrend eligepisode) force;
						run;
				%END;

				%IF &INCLUSION.=0 %THEN %DO;
					proc append base=_Denom&Group._STACK_EXCL
						data=_Denom&Group._&cond.(keep=patid denomenrstart denomenrend eligepisode) force;
					    run;
				%END;

			%END;*End Loop Cond;

*Output dataset for inclusion=1 and for exclusions=0 and then (a) shaveoutside (b) shaveinside denom; 
*For inclusions, need to merge periods;

			*First shaveoutside denom file but need to merge inclusion periods together;
			%IF %SYSFUNC(exist(_Denom&Group._STACK_INC))=1 %THEN %DO;
				%MergeTimePeriods(dataset=_Denom&Group._STACK_INC,
						          DateStart=DenomEnrStart,
						          DateEnd=DenomEnrEnd);


        *_Denom&Group._STACK_INC includes all time periods where the potential index dates will meet at least one of the cond inclusions;
				data _Denom&Group._STACK_INC;
				set _Denom&Group._STACK_INC;
				rename 
				DenomEnrStart=DTSTART
				DenomEnrEnd=DTEND
				;
				run;

				%ms_shaveoutside(reffile=_Denom&Group._STACK_INC,
							   	 refstart=DTSTART,
						       	 refend=DTEND,  
							   	 /*id=EligEpisode,*/ 
							   	 KeepPartBf=Y,
						       	 ToShaveFile=_Denom&Group.,
						       	 ToShaveStart=DenomEnrStart,
						       	 ToShaveEnd=DenomEnrEnd,
						       	 outfile=_Denom&Group./*_incl*/);	

			%END;

			%IF %SYSFUNC(exist(_Denom&Group._STACK_EXCL))=1 %THEN %DO;

        *merge all periods of ineligibilities;
				%MergeTimePeriods(dataset=_Denom&Group._STACK_EXCL,
						          DateStart=DenomEnrStart,
						          DateEnd=DenomEnrEnd);

      *_Denom&Group._STACK_EXCL includes all time periods where the potential index dates will meet at least one of the cond exclusions;
				data _Denom&Group._STACK_EXCL;
				set _Denom&Group._STACK_EXCL;
				rename 
				DenomEnrStart=DTSTART
				DenomEnrEnd=DTEND
				;
				run;


        *remove all these periods from the ELIG;
				%ms_shaveinside(reffile=_Denom&Group.,
					           	refstart=DenomEnrStart,
					            refend=DenomEnrEnd,  
								id=EligEpisode, 				    
					            ToShaveFile=_Denom&Group._STACK_EXCL,
					            ToShaveStart=DTSTART,
					            ToShaveEnd=DTEND,
					            outfile=_Denom&Group./*_excl*/);
			%END;



			proc sort data=_Denom&Group. out=_Denom&Group. nodupkey;						
			by patid denomenrstart denomenrend eligepisode;
			run;

		%END;*If there are inclusion codes for GROUP;
	%END;*If there are inclusion codes;
%mend wrapper;
%wrapper;

/*---------------------------------------------------*/
/* Group Index (POV1) to remove from eligibility	 */
/*---------------------------------------------------*/
%macro wrapper;
	%if %eval(&itt.=1) %then %do;
		%let InDataset=_groupindexitt;
    	data _groupindexitt;
		set _groupindexitt;
		ExpireDt=ADate+RxSup-1;
		run;
	%end;
	%else %do;
		%let InDataset=_groupindex;
	%end;

	%ISDATA(dataset=&InDataset.);
	%IF %EVAL(&NOBS.>=1) & %EVAL(&WASHPER. ne 0)  %THEN %DO;
		data _UneligGroupIndex;
		set &InDataset.;
		format UneligStart UneligEnd date9.;
		UneligStart=ADate + 1;	* Added a +1 so a member can remain eligible the day of index date;
		UneligEnd=ExpireDt + min(&WASHPER.,99999);
        if UneligStart<=UneligEnd;
		keep PatId UneligStart UneligEnd;
		run;

		%MergeTimePeriods(dataset=_UneligGroupIndex,
		                  DateStart=UneligStart,
		                  DateEnd=UneligEnd);
		%ms_shaveinside(reffile=_Denom&Group.,
		                refstart=DenomEnrStart,
		                refend=DenomEnrEnd,  
					          id=EligEpisode, 
		                ToShaveFile=_UneligGroupIndex,
		                ToShaveStart=UneligStart,
		                ToShaveEnd=UneligEnd,
		                outfile=_Denom&Group.);
	%END;
%mend wrapper;
%wrapper;

/*---------------------------------------------------*/
/* Group Washout (POV2) to remove from eligibility	 */
/*---------------------------------------------------*/
%macro wrapper;
	%ISDATA(dataset=_groupwash);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;

		data _UneligGroupWash;
		set _groupwash;
		format UneligStart UneligEnd date9.;
		UneligStart=ADate + 1;	* Added a +1 so a member can remain eligible the day of index date;
		UneligEnd=ExpireDt + min(&WASHPER.,99999);
        if UneligStart<=UneligEnd;
		keep PatId UneligStart UneligEnd;
		run;

		%MergeTimePeriods(dataset=_UneligGroupWash,
		                  DateStart=UneligStart,
		                  DateEnd=UneligEnd);
		%ms_shaveinside(reffile=_Denom&Group.,
		                refstart=DenomEnrStart,
		                refend=DenomEnrEnd,  
					    id=EligEpisode, 
		                ToShaveFile=_UneligGroupWash,
		                ToShaveStart=UneligStart,
		                ToShaveEnd=UneligEnd,
		                outfile=_Denom&Group.);
	%END;
%mend wrapper;
%wrapper;


/*---------------------------------------------------*/
/* Follow-up Event (POV5) to remove from eligibility */
/*---------------------------------------------------*/
%macro wrapper;
	%ISDATA(dataset=_fupevent);*Will only apply to type 2 analysis;
	%IF %EVAL(&NOBS.>=1) %THEN %DO;
		data _UneligFupEvent;
		set _fupevent;
		format UneligStart UneligEnd date9.;
		UneligStart=sum(ADate, - &BLACKOUTPER., 1);	* Added a +1 so a member can be eligible the day of event;
		UneligEnd=ExpireDt + min(&FUPWASHPER.,99999);
        if UneligStart<=UneligEnd;
		keep PatId UneligStart UneligEnd;
		run;

		%MergeTimePeriods(dataset=_UneligFupEvent,
		                  DateStart=UneligStart,
		                  DateEnd=UneligEnd);
		%ms_shaveinside(reffile=_Denom&Group.,
		                refstart=DenomEnrStart,
		                refend=DenomEnrEnd,  
					    id=EligEpisode, 
		                ToShaveFile=_UneligFupEvent,
		                ToShaveStart=UneligStart,
		                ToShaveEnd=UneligEnd,
		                outfile=_Denom&Group.);
	%END;
%mend wrapper;
%wrapper;


/*-----------------------------------------------------*/
/* Follow-up Washout (POV6) to remove from eligibility */
/*-----------------------------------------------------*/
%macro wrapper;
	%ISDATA(dataset=_fupwash);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;

		data _UneligFupWash;
		set _fupwash;
		format UneligStart UneligEnd date9.;
		UneligStart=ADate + 1;	* Added a +1 so a member can remain eligible the day of index date;
		UneligEnd=ExpireDt + min(&FUPWASHPER.,99999);
        if UneligStart<=UneligEnd;
		keep PatId UneligStart UneligEnd;
		run;

		%MergeTimePeriods(dataset=_UneligFupWash,
		                  DateStart=UneligStart,
		                  DateEnd=UneligEnd);
		%ms_shaveinside(reffile=_Denom&Group.,
		                refstart=DenomEnrStart,
		                refend=DenomEnrEnd,  
					    id=EligEpisode, 
		                ToShaveFile=_UneligFupWash,
		                ToShaveStart=UneligStart,
		                ToShaveEnd=UneligEnd,
		                outfile=_Denom&Group.);
	%END;
%mend wrapper;
%wrapper;


/*---------------------------------------------------*/
/* Remove ITT episode lengths from eligibility -     */ 
/* patient no longer eligible to become incidence    */
/* until ITT has expired                             */
/*---------------------------------------------------*/
%macro wrapper;
	%if %eval(&itt.=1) %then %do;
		data _UneligITT;
		set _ptsmasterlist;
		format UneligStart UneligEnd date9.;
		UneligStart=IndexDt + 1;
		UneligEnd=IndexDt + &ITTDays. - 1;
        if UneligStart<=UneligEnd;
		keep PatId UneligStart UneligEnd;
		run;

		%ISDATA(dataset=_UneligITT);
		%IF %EVAL(&NOBS.>=1) %THEN %DO;
	        %ms_shaveinside(reffile=_Denom&Group.,
	                        refstart=DenomEnrStart,
	                        refend=DenomEnrEnd,  
						    id=EligEpisode, 
	                        ToShaveFile=_UneligITT,
	                        ToShaveStart=UneligStart,
	                        ToShaveEnd=UneligEnd,
	                        outfile=_Denom&Group.);
	    %END;
	%end;
%mend wrapper;
%wrapper;

/*-----------------------------------------------------*/
/* Make sure the number of enrollment days remaining   */
/* after each potential elig days is sufficient		   */
/*-----------------------------------------------------*/
* Extract highest CONDTO value among exclusions if any;
%let MaxExclCondTo=0;
%macro wrapper;
    %ISDATA(dataset=&INCLUSIONCODES.);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;
		proc sql noprint;
		select max(condto) into :MaxExclCondTo 
    	from &INCLUSIONCODES.(where=(group="&ITGROUP.")) 
		where condinclusion=0;
		quit;
	%END;
	%put &MaxExclCondTo.;
%mend wrapper;
%wrapper;

* Get first Index Dates and Event Dates to handle member days correctly according to CohortDef;
    proc sql noprint;
    create table _Denom as 
    select den.PatId,
    	   den.EligEpisode,
    	   den.Enr_Start,
    	   den.Enr_End,
    	   min(lst.IndexDt) as FIndexDt format date9.,
    	   min(lst.FEventDt) as FEventDt format date9.
    from _DenomOrig&Group. as den
    left join _ptsmasterlist as lst
    on den.PatId=lst.PatId
    group by den.PatId, den.EligEpisode, den.Enr_Start, den.Enr_end
    order by PatId, EligEpisode;
    quit;

    proc sort data = _Denom&Group.;
    by PatId EligEpisode;
    run;

    data _Denom&Group.;
    merge _Denom&Group.(in=a keep=PatId EligEpisode DenomEnrStart DenomEnrEnd)
    	  _Denom;
    by PatId EligEpisode;
    format AdjustedEnrEnd date9.;
    if a;

    * Adjust eligibility end date;
    AdjustedEnrEnd = Enr_End - Max(0,&MinEpisDur. - 1,&MinDaySupp. - 1,&BlackoutPer.,&PRIORDAYS_AF.,&MaxExclCondto.,&MAXCOVTO.,&L1HDPSTO.);
    DenomEnrEnd = min(DenomEnrEnd,AdjustedEnrEnd);

    if "&COHORTDEF."="01" then DenomEnrEnd=min(DenomEnrEnd,FIndexDt);
    if "&COHORTDEF."="03" then DenomEnrEnd=min(DenomEnrEnd,FEventDt);

    MemberDays = DenomEnrEnd - DenomEnrStart + 1;

    * Because DenomEnrEnd can have been truncated, keep only the observations with positive member days;
    if MemberDays>0;
    run;
/*-------------*/
/* Add DOB/SEX */
/*-------------*/

	proc sort nodupkey data=_Denom&Group. out=_UniquePts(Keep=PatId);
	by PatId;
	run;

    proc sql noprint;
        /*Age and demogs*/		
		create table _DemoCount as
        select Dem.PatId, 
			   Count(Dem.PatId) as numdemo
        from indata.&demtable. as DEM 
             inner join 
             _UniquePts as pts
        on DEM.patid = pts.patid
		group by Dem.PatId;

        *restrict to unique pts;
        create table _Demo as
        select distinct pts.*, 
                        BIRTH_DATE, 
                        SEX,
						zip,
						zip_date
        from indata.&demtable as DEM 
             inner join 
             _Denom&Group. as pts			 
        on DEM.patid = pts.patid
			 inner join _DemoCount (where=(numdemo=1)) as cnt
		on cnt.patid = pts.patid;

		create table  _Denom&Group.
		as select * 
		from _demo;        
    quit;

		/** Zip Codes **/
        %ISDATA(dataset=infolder.&ZIPFILE.);	
        %IF %EVAL(&NOBS.>0) %THEN %DO;
			proc sql noprint;
				create table _zip&Group. as
				select pts.*,
					 lkup.statecode 
				from _Denom&Group. as pts
				left join infolder.&zipfile. as lkup
				on pts.zip = lkup.zip
				order by pts.patid;
			quit;

			data _Denom&Group.(drop=statecode);
				length zip3 state $7;
				set _zip&Group.;
					if missing(zip) then do;
						zip3 = 'Missing';
						state = 'Missing';
					end;
					else if missing(statecode) then do;
						zip3 = 'Invalid';
						state = 'Invalid';
					end;
					else do;
						zip3 = substr(strip(zip),1,3);
						state = statecode;
					end;
			run;
		%END;

*truncate periods such that all interval lies in Minage-MaxAge;

    data _Denom&Group.;
    set _Denom&Group.;

    format MinAgeDate  MaxAgeDate date9.;
    MinAgeDate = intnx(scan("&AGETYP.",1),birth_date,&MINAGE.,'sameday');

    if &MAXAGE.=99999 then MaxAgeDate=intnx('Years',birth_date,110,'sameday');
    else MaxAgeDate= intnx(scan("&AGETYP.",&NBSTRAT.),birth_date,&MAXAGE.+1,'sameday'); 

    *Member not meetin minimum age yet;
    if MinAgeDate > DenomEnrStart then DenomEnrStart=MinAgeDate;  *can be pushed beyond DenomEnrEnd;

    *Member has reached or will reach will reach maxage ;
    if MaxAgeDate <= DenomEnrEnd then DenomEnrEnd=MaxAgeDate-1;   *can be pushed before DenomEnrStart;
    
    *Adjust;
    if DenomEnrStart <= DenomEnrEnd;

    run;


*At this point, each day in the DenomEnrStart-DenomEnrEnd intervals represents
 a potential index date if the patients would have started one of the study drugs.
 Now, we need to restrict only to the startfollowup-enddate period. This will simplify accumulating days
 for year and year/Month strata, especially if the startfollowup day is not the first day of the month
 and or the first month of the year;

    data _Denom&Group.;
    set _Denom&Group.; 
    sfup=input("&startfollowup.", best.);
    format sfup date9.;
    DenomEnrStart=max(DenomEnrStart,&startfollowup.);
    DenomEnrEnd=min(DenomEnrEnd,&enddate.);
    patient=1;
    MemberDays=DenomEnrEnd-DenomEnrStart+1;
    if DenomEnrStart<=DenomEnrEnd;
    run;


*Prepare dates for year/month loop;
    %global nummonths;
    %global numyears;
    data _null_;
    format startfollowup enddate date9.;
    startfollowup=intnx('month',&startfollowup.,0,'begin');  *in case not at the start of month;
    enddate=intnx('month',&enddate.,1,'begin')-1; *in case not at the end of month;
    nummonths=intck('month',startfollowup,enddate)+1;
    numyears=intck('year',startfollowup,enddate)+1;

    call symputx("StartYear",year(startfollowup));
    call symputx("StartMonth",month(startfollowup));
    call symputx("EndYear",year(enddate));
    call symputx("EndMonth",month(enddate));
    call symputx("nummonths",nummonths);
    call symputx("numyears",numyears);

    do i=1 to nummonths+1;
        call symput("PER"||strip(put(i,best.)),put(intnx('month',&startfollowup.,i-1,'begin'),best.));
    end;
    do i=1 to numyears+1;
        call symput("YEAR"||strip(put(i,best.)),put(intnx('year',&startfollowup.,i-1,'begin'),best.));
    end;

    run;
    %PUT &StartYear. &StartMonth. &EndYear. &EndMonth. &nummonths. &numyears.;
    
    *this macro is to avoid having to output one record per interval per stratification level
     which for certain partners, can become quite large.;
options nosymbolgen;

    %macro CreateSums(infile=,outfile=,by=,dimout=);
        data _temp;
        set &infile.;
        By &by. PatId;

        array _NumPtsYM(*) _NumPtsYM1-_NumPtsYM&nummonths.;   *Pts level temporary vector to avoid double counting patients in same year/month;   
        array NumPtsYM(*) NumPtsYM1-NumPtsYM&nummonths.;             *accumator of unique patient in the same year/month;
        array NumMemDaysYM(*) NumMemDaysYM1-NumMemDaysYM&nummonths.; *accumator of unique days in the same year/month;

        array _NumPtsY(*) _NumPtsY1-_NumPtsY&numyears.;   *Pts level temporary vector to avoid double counting patients in same year/month;   
        array NumPtsY(*) NumPtsY1-NumPtsY&numyears.;                 *accumator of unique patient in the same year;
        array NumMemDaysY(*) NumMemDaysY1-NumMemDaysY&numyears.;     *accumator of unique days in the same year;

        %do p=1 %to &nummonths.;
            %let p1=%eval(&p.+1);
            if %MS_PeriodsOverlap(period1=DenomEnrStart DenomEnrEnd,period2=&&PER&p. &&PER&p1.-1) then do;
                _NumPtsYM(&p.)=1;
                NumMemDaysYM(&p.)=sum(NumMemDaysYM(&p.),Min(DenomEnrEnd,&&PER&p1.-1)-max(DenomEnrStart,&&PER&p.)+1);
            end;                      
        %end;

        %do p=1 %to &numyears.;
            %let p1=%eval(&p.+1);
            if %MS_PeriodsOverlap(period1=DenomEnrStart DenomEnrEnd,period2=&&YEAR&p. &&YEAR&p1.-1) then do;
                _NumPtsY(&p.)=1;
                NumMemDaysY(&p.)=sum(NumMemDaysY(&p.),Min(DenomEnrEnd,&&YEAR&p1.-1)-max(DenomEnrStart,&&YEAR&p.)+1);
            end;                      
        %end;

 
        if last.PatId then do;
            do i = 1 to &NUMYEARS.;               
                NumPtsY(i)=sum(NumPtsY(i),_NumPtsY(i));
            end;
            do i = 1 to &nummonths.;
                 NumPtsYM(i)=sum(NumPtsYM(i),_NumPtsYM(i));
            end; 
            call missing(of _Num:);
        end;

        if last.&dimout. then do;
            output;
            call missing(of Num:);
        end;

        keep &by. Num:;

        retain Num: _Num:;
        run;

	%ISDATA(dataset=_temp);
	%IF %EVAL(&NOBS.=0) %THEN %DO;
		proc sql noprint;
			insert into _temp set NumPtsYM1=.;			
		quit;		
	%END;

    proc transpose data=_temp(drop=NumPts:) out=_Days prefix=NumMemDays;
    by &by.;
    run;
    proc transpose data=_temp(drop=NumMemDays:) out=_PTS prefix=NumPts;
    by &by.;
    run;

	/* Get system default value; store in macro variable of same name */
 	%let mergenoby=%sysfunc(getoption(mergenoby));
	/* Set mergenoby to nowarn to avoid triggering an error or warning at this step */
	options mergenoby=nowarn;

    data &outfile.;
    merge _Days(rename=_NAME_=_Days_)
          _PTS(rename=_NAME_=_PTS_);
    *No by;

    if _PTS_=:"NumPtsYM" then do;
        per=input(tranwrd(_PTS_, 'NumPtsYM' , ''),best.);
        date=intnx('month',&startfollowup.,per-1,'begin');
        month=month(date);
        year=year(date);
    end;
    else do;
        per=input(tranwrd(_PTS_, 'NumPtsY' , ''),best.);
        date=intnx('year',&startfollowup.,per-1,'begin');
        year=year(date);
    end;

    Rename NumMemDays1=DenNumMemDays
           NumPts1=DenNumPts;

    keep  year month &by. Num:;
    run;
	
	/* Reset mergenoby to original value (as stored in macro variable) */
 	options mergenoby=&mergenoby.;

    %mend CreateSums;


*create individual denom contribution to final table;
*need to do this individually to make sur we dont multiple-count
*patients in the presence of different strata;

    *Create Overall counts;
    proc means data=_Denom&Group. nway noprint;
    var MemberDays Patient;
    class PatId;
    output out=_ALL sum(MemberDays)= max(Patient)=;
    run;
    proc means data=_ALL noprint;
    var MemberDays Patient;
    output out=_ALL(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
    run;

    proc sort data=_Denom&Group.;
    By Sex PatId;
    run;

    *Create sex level counts;
    proc means data=_Denom&Group. noprint;
    var MemberDays Patient;
    by Sex PatId;
    output out=_S sum(MemberDays)= max(Patient)=;
    run;
    proc means data=_S noprint;
    var MemberDays Patient;
    by Sex;
    output out=_S(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
    run;

    *Sex Year and Sex Year Month;
    %CreateSums(infile=_Denom&Group.,
                outfile=_SYM,
                by=sex,
                dimout=Sex);

    *year only (will use _SYM as sex is mutually exclusive;
    proc means data=_SYM nway noprint;
    var DenNumMemDays DenNumPts;
    class year;
    where year ne . and month=.;
    output out=_Y (drop=_:) sum(DenNumMemDays)=DenNumMemDays sum(DenNumPts)=DenNumPts;
    run;

	*year month only (will use _SYM as sex is mutually exclusive;
    proc means data=_SYM nway noprint;
    var DenNumMemDays DenNumPts;
    class year month;
    where year ne . and month ne .;
    output out=_YM (drop=_:) sum(DenNumMemDays)=DenNumMemDays sum(DenNumPts)=DenNumPts;
    run;

/*-------------------------------------------------*/
/* further breaking down periods by age categories */
/*-------------------------------------------------*/
    data _Denom&Group.Age(rename=AgeDenomEnrEnd=DenomEnrEnd rename=AgeDenomEnrStart=DenomEnrStart);
    set _Denom&Group.;
    format AgeGroup $9.; 
    format threshdate AgeDenomEnrStart AgeDenomEnrEnd date9.;
    AgeDenomEnrStart=DenomEnrStart;
    AgeDenomEnrEnd=DenomEnrEnd;
    *OrigMemberDays=DenomEnrEnd-DenomEnrStart+1;
    do i=&NBSTRAT. to 1 by -1;       
    threshdate=intnx(scan("&AGETYP.",i),birth_date,input(scan("&AGETHRESH.",i),best.),'sameday'); 
    if threshdate <= AgeDenomEnrEnd then do;
       AgeGroup = scan("&AGESTRAT.",i,' ');
       AgeGroupSort = i;
       AgeDenomEnrStart=max(threshdate,DenomEnrStart);
       MemberDays=AgeDenomEnrEnd-AgeDenomEnrStart+1;
       output;
       AgeDenomEnrEnd=threshdate-1;
       if AgeDenomEnrEnd < DenomEnrStart then leave;
    end;
    end;         
    keep PatId AgeDenomEnrEnd AgeDenomEnrStart sex AgeGroup AgeGroupSort Patient MemberDays threshdate; 
    run;
   

    *AgeGroup level counts;
    proc means data=_Denom&Group.Age  nway noprint;
    var MemberDays Patient;
    class AgeGroupSort PatId;
    output out=_AG sum(MemberDays)= max(Patient)=;
    run;
    proc means data=_AG nway noprint;
    var MemberDays Patient;
    class AgeGroupSort;
    output out=_AG(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
    run;

    proc sort data=_Denom&Group.Age;
    By Sex AgeGroupSort PatId;
    run;

    *Sex AgeGroup level counts;
    proc means data=_Denom&Group.Age noprint;
    var MemberDays Patient;
    by Sex AgeGroupSort PatId;
    output out=_SAG sum(MemberDays)= max(Patient)=;
    run;
    proc means data=_SAG  noprint;
    var MemberDays Patient;
    by Sex AgeGroupSort;
    output out=_SAG(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
    run;

    *Sex AgeGroup year and Sex AgeGroup year month level counts;
    %CreateSums(infile=_Denom&Group.Age,
                outfile=_SAGYM,
                by=Sex AgeGroupSort,
                dimout=AgeGroupSort);

    proc sort data=_Denom&Group.Age;
    By AgeGroupSort PatId;
    run;

    *AgeGroup year and AgeGroup year month level counts;
    %CreateSums(infile=_Denom&Group.Age,
                outfile=_AGYM,
                by=AgeGroupSort,
                dimout=AgeGroupSort);

/*-------------------------------------------------*/
/* Geographic categories						   */
/*-------------------------------------------------*/

	%macro geogdenoms(table= , vars=);
		%let prefix = %substr(&table,1,3);

	  *Create Overall counts;
	    proc means data=_Denom&Group. nway noprint;
	    var MemberDays Patient;
	    class PatId;
	    output out=_ALL&table. sum(MemberDays)= max(Patient)=;
	    run;
	    proc means data=_ALL&table. noprint;
	    var MemberDays Patient;
	    output out=_ALL&table.(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
	    run;

		*Create State/Zip level counts;
		 proc sort data=_Denom&Group.;
		    By &vars. PatId;
		    run;

		    proc means data=_Denom&Group. noprint;
		    var MemberDays Patient;
		    by &vars. PatId;
		    output out=_&prefix. sum(MemberDays)= max(Patient)=;
		    run;
		    proc means data=_&prefix. noprint;
		    var MemberDays Patient;
		    by &vars.;
		    output out=_&prefix.(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
		    run;

		*create sex level counts;
		 proc sort data=_Denom&Group.;
		    By &vars. Sex PatId;
		    run;

		    proc means data=_Denom&Group. noprint;
		    var MemberDays Patient;
		    by &vars. sex PatId;
		    output out=_&prefix._S sum(MemberDays)= max(Patient)=;
		    run;
		    proc means data=_&prefix._S noprint;
		    var MemberDays Patient;
		    by &vars. sex;
		    output out=_&prefix._S(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
		    run;

		*Create Age Group level counts;
		  data _Denom&Group.Age&table.(rename=AgeDenomEnrEnd=DenomEnrEnd rename=AgeDenomEnrStart=DenomEnrStart);
		    set _Denom&Group.;
		    format AgeGroup $9.; 
		    format threshdate AgeDenomEnrStart AgeDenomEnrEnd date9.;
		    AgeDenomEnrStart=DenomEnrStart;
		    AgeDenomEnrEnd=DenomEnrEnd;
		    *OrigMemberDays=DenomEnrEnd-DenomEnrStart+1;
		    do i=&NBSTRAT. to 1 by -1;       
		    threshdate=intnx(scan("&AGETYP.",i),birth_date,input(scan("&AGETHRESH.",i),best.),'sameday'); 
		    if threshdate <= AgeDenomEnrEnd then do;
		       AgeGroup = scan("&AGESTRAT.",i,' ');
		       AgeGroupSort = i;
		       AgeDenomEnrStart=max(threshdate,DenomEnrStart);
		       MemberDays=AgeDenomEnrEnd-AgeDenomEnrStart+1;
		       output;
		       AgeDenomEnrEnd=threshdate-1;
		       if AgeDenomEnrEnd < DenomEnrStart then leave;
		    end;
		    end;         
		    keep PatId AgeDenomEnrEnd AgeDenomEnrStart sex &vars. AgeGroup AgeGroupSort Patient MemberDays threshdate; 
		    run;

			proc sort data=_Denom&Group.Age&table.; 
				by &vars. AgeGroupSort PatId;
			run;
    
		    proc means data=_Denom&Group.Age&table. noprint;
		    var MemberDays Patient;
		    by &vars. AgeGroupSort PatId;
		    output out=_&prefix._AG sum(MemberDays)= max(Patient)=;
		    run;

		    proc means data=_&prefix._AG  noprint;
		    var MemberDays Patient;
		    by &vars. AgeGroupSort;
		    output out=_&prefix._AG(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
		    run;
		
		*Create Year level counts (using %createsums in order to preserve future option to stratify by month);
			proc sort data=_Denom&Group.; 
				by &vars. PatId;
			run;

		    %CreateSums(infile=_Denom&Group.,
		                outfile=_&prefix._Y,
		                by=&vars.,
		                dimout=&table.);

		    proc means data=_&prefix._Y nway noprint;
		    var DenNumMemDays DenNumPts;
		    class &vars. year;
		    where year ne . and month=.;
		    output out=_&prefix._Y (drop=_:) sum(DenNumMemDays)=DenNumMemDays sum(DenNumPts)=DenNumPts;
		    run;


		/*Need to adjust denominator for zip_date*/
		    data _Denom&Group.&table.(rename=ZipDenomEnrEnd=DenomEnrEnd rename=ZipDenomEnrStart=DenomEnrStart);
			    set _Denom&Group.;
	   
			    format ZipDenomEnrStart ZipDenomEnrEnd date9.;
				ZipDenomEnrStart=DenomEnrStart;
				ZipDenomEnrEnd=DenomEnrEnd;

				/*1. if zip_date < DenomEnrStart, full enrollment is certain*/
				if . < zip_date < DenomEnrStart then do;
					MemberDays=DenomEnrEnd-DenomEnrStart+1;
					zip_uncertain = 'N';
					output;
				end;
				/*2. if zip_date is missing, full enrollment is uncertain*/
				else if zip_date = . then do;
					MemberDays=DenomEnrEnd-DenomEnrStart+1;
					zip_uncertain = 'Y';
					output;
				end;
				/*3. if zip_date > ZipEnrEnd, then no uncertain time*/ 
				else if zip_date >= DenomEnrEnd then do;
					MemberDays=DenomEnrEnd-DenomEnrStart+1;
					zip_uncertain = 'Y';
					output;
				end;
				/*4. if DenomEnrStart <= zip_date <= DenomEnrEnd, then calculate uncertain time*/ 
				else do;
			       	zip_uncertain = 'Y';
				   	/*Adjust Denom enrollment*/
				   	ZipDenomEnrEnd=zip_date;
			       	MemberDays=zip_date-DenomEnrStart+1;
			       	output;

       				zip_uncertain = 'N';
				   	ZipDenomEnrStart=zip_date+1;
					ZipDenomEnrEnd = DenomEnrEnd;
				   	MemberDays=ZipDenomEnrEnd-ZipDenomEnrStart; /*no +1 because member is certain on zip_date*/
					output;
			    end;       
			    keep PatId ZipDenomEnrEnd ZipDenomEnrStart &vars. sex Patient MemberDays zip_uncertain zip_date birth_date; 
		    run;
		   
		*create uncertain level counts;
		 proc sort data=_Denom&Group.&table.;
		    By &vars. zip_uncertain PatId;
		 run;

		 proc means data=_Denom&Group.&table. noprint;
		    var MemberDays Patient;
		    by &vars. zip_uncertain PatId;
		    output out=_&prefix._U sum(MemberDays)= max(Patient)=;
		 run;
		 proc means data=_&prefix._U noprint;
		    var MemberDays Patient;
		    by &vars. zip_uncertain;
		    output out=_&prefix._U(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
		 run;
		   
		*create uncertain*sex level counts;
		 proc sort data=_Denom&Group.&table.;
		    By &vars. zip_uncertain sex PatId;
		 run;

		 proc means data=_Denom&Group.&table. noprint;
		    var MemberDays Patient;
		    by &vars. zip_uncertain sex PatId;
		    output out=_&prefix._U_S sum(MemberDays)= max(Patient)=;
		 run;
		 proc means data=_&prefix._U_S noprint;
		    var MemberDays Patient;
		    by &vars. zip_uncertain sex;
		    output out=_&prefix._U_S(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
		 run;
		   
		*Create age group*uncertain counts;
		  data _Denom&Group.Age&table._U(rename=AgeDenomEnrEnd=DenomEnrEnd rename=AgeDenomEnrStart=DenomEnrStart);
				    set _Denom&Group.&table.;
				    format AgeGroup $9.; 
				    format threshdate AgeDenomEnrStart AgeDenomEnrEnd date9.;
				    AgeDenomEnrStart=DenomEnrStart;
				    AgeDenomEnrEnd=DenomEnrEnd;
				    *OrigMemberDays=DenomEnrEnd-DenomEnrStart+1;
				    do i=&NBSTRAT. to 1 by -1;       
				    threshdate=intnx(scan("&AGETYP.",i),birth_date,input(scan("&AGETHRESH.",i),best.),'sameday'); 
				    if threshdate <= AgeDenomEnrEnd then do;
				       AgeGroup = scan("&AGESTRAT.",i,' ');
				       AgeGroupSort = i;
				       AgeDenomEnrStart=max(threshdate,DenomEnrStart);
				       MemberDays=AgeDenomEnrEnd-AgeDenomEnrStart+1;
				       output;
				       AgeDenomEnrEnd=threshdate-1;
				       if AgeDenomEnrEnd < DenomEnrStart then leave;
				    end;
				    end;         
				    keep PatId AgeDenomEnrEnd AgeDenomEnrStart sex &vars. AgeGroup AgeGroupSort Patient MemberDays threshdate zip_uncertain; 
				    run;

					proc sort data=_Denom&Group.Age&table._U; 
						by &vars. zip_uncertain AgeGroupSort PatId;
					run;
		    
				    proc means data=_Denom&Group.Age&table._U noprint;
				    var MemberDays Patient;
				    by &vars. zip_uncertain AgeGroupSort PatId;
				    output out=_&prefix._U_AG sum(MemberDays)= max(Patient)=;
				    run;
				    proc means data=_&prefix._U_AG  noprint;
				    var MemberDays Patient;
				    by &vars. zip_uncertain AgeGroupSort;
				    output out=_&prefix._U_AG(drop=_:) sum(MemberDays)=DenNumMemDays sum(Patient)=DenNumPts;
				    run;

		*Create Year*uncertain level counts;
			proc sort data=_Denom&Group.&table.;
				by &vars. zip_uncertain PatId;
			run;

		    %CreateSums(infile=_Denom&Group.&table.,
		                outfile=_&prefix._U_Y,
		                by=&vars. zip_uncertain,
		                dimout=zip_uncertain);

		    proc means data=_&prefix._U_Y nway noprint;
		    var DenNumMemDays DenNumPts;
		    class &vars. zip_uncertain year;
		    where year ne . and month=.;
		    output out=_&prefix._U_Y (drop=_:) sum(DenNumMemDays)=DenNumMemDays sum(DenNumPts)=DenNumPts;
		    run;
	%mend;

	%if %index(&GEOG.,ZIP3) > 0 %then %do;
		%geogdenoms(table=zip3, vars=zip3);
		    data _DenomCountsZIP3;
			 set _ALLzip3(in=a)
		        _ZIP(in=b)
		        _ZIP_U(in=c)
		        _ZIP_S(in=d)
				_ZIP_U_S(in=e)
		        _ZIP_AG(in=f)
		        _ZIP_U_AG(in=g)
		        _ZIP_Y(in=h)
				_ZIP_U_Y(in=i);
		    format group $30.;
		    group="&itgroup.";
		    if a then level="000";
		    if b then level="020";
		    if c then level="021";
		    if d then level="022";
		    if e then level="023";
		    if f then level="024";
		    if g then level="025";
		    if h then level="026";
			if i then level="027";
		    run;

			*Store patient denoms to DPLocal;
		    %IF %EVAL(&group.=1) %THEN %DO;
		        data DPLocal.&RUNID._DenomCountsZip3;
		        retain Level group zip3 zip_uncertain sex agegroupsort year DenNumPts DenNumMemDays;
		        set _DenomCountsZIP3;
		        run;
		    %END;
		    %ELSE %DO;
		        proc append base=DPLocal.&RUNID._DenomCountsZip3 
		                    data=_DenomCountsZIP3 force;
		        run;
		    %END;
	%end;
	%if %index(&GEOG.,STATE) > 0 %then %do;
		%geogdenoms(table=state,vars=state);
			data _DenomCountsSTATE;
			 set _ALLstate(in=a)
		        _STA(in=b)
		        _STA_U(in=c)
		        _STA_S(in=d)
				_STA_U_S(in=e)
		        _STA_AG(in=f)
		        _STA_U_AG(in=g)
		        _STA_Y(in=h)
				_STA_U_Y(in=i);
		    format group $30.;
		    group="&itgroup.";
		    if a then level="000";
		    if b then level="030";
		    if c then level="031";
		    if d then level="032";
		    if e then level="033";
		    if f then level="034";
		    if g then level="035";
		    if h then level="036";
			if i then level="037";
		    run;		

			*Store patient denoms to DPLocal;
		    %IF %EVAL(&group.=1) %THEN %DO;
		        data DPLocal.&RUNID._DenomCountsSTATE;
		        retain Level group state zip_uncertain sex agegroupsort year DenNumPts DenNumMemDays;
		        set _DenomCountsSTATE;
		        run;
		    %END;
		    %ELSE %DO;
		        proc append base=DPLocal.&RUNID._DenomCountsSTATE 
		                    data=_DenomCountsSTATE force;
		        run;
		    %END;
	%end;

*combining all counts;
* relevant denominator counts are
    Overall (_ALL)
    Year(_Y)
    Sex (_S)
    AgeGroup (_AG)
    Sex AgeGroup (_SAG)
    Sex AgeGroup Year (_SAGYM)
    Sex AgeGroup Year Month (_SAGYM)
    AgeGroup Year (_AGYM)
    AgeGroup Year Month (_AGYM)
    Sex Year(_SYM)
    Sex Year Month (_SYM);

    data _DenomCounts;
    set _ALL(in=a)
        _Y(in=b)
        _S(in=c)
        _AG(in=d)
        _SAG(in=e)
        _SAGYM(in=f)
        _AGYM(in=g)
        _SYM(in=h)
		_YM(in=i);
    format group $30.;
    group="&itgroup.";
    if a then level="000";
    if b then level="001";
    if c then level="002";
    if d then level="003";
    if e then level="004";
    if f and month eq . then level="005";
    if f and month ne . then level="006";
    if g and month eq . then level="007";
    if g and month ne . then level="008";
    if h and month eq . then level="009";
    if h and month ne . then level="010";
	if i then level="011";
    run;

	*Store patient denoms to DPLocal;
    %IF %EVAL(&group.=1) %THEN %DO;
        data DPLocal.&RUNID._DenomCounts;
        retain Level group sex agegroupsort year month DenNumPts DenNumMemDays;
        set _DenomCounts;
        run;
    %END;
    %ELSE %DO;
        proc append base=DPLocal.&RUNID._DenomCounts 
                    data=_DenomCounts force;
        run;
    %END;


%put NOTE: ******** END OF MACRO: ms_cidadenom v1.7 ********;

%mend ms_cidadenom;
