
****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidatablest4.sas  
*
* Created (mm/dd/yyyy): 08/24/2015
* Last modified: 08/10/2016
* Version: 1.1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create characteristics and medication of interest tables for pregnancy analyses.                                        
*   
*  Program inputs:  
*	- a dataset containing MOI drug codes 
*	- a dataset containing pregnancy codes
*	- a dataset containing defined group pregnancy and control episodes
*	- a dataset containing MOI claims
*             	                        
*  Program outputs:                                                                                                                                       
*	- Characteristics and Medication of Interest tables
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    1.1	08/10/2016   AP (SOC)	Added underscore prefix to proc transpose (QCI-179)
*
***************************************************************************************************;

%macro ms_cidatablest4();

	%put =====> MACRO CALLED: ms_cidatablest4 v1.0;

    *Remove output tables if they are present in MSOC because the append function will be used below;
    proc datasets library=MSOC nolist nowarn;
    delete &RUNID._chars:;
    delete &RUNID._meduse:;
    delete &RUNID._EpisodesByEnrol;
    quit;

	*Get the total number of categories to process;
	proc contents data=infolder.&MOICODES.(keep=Category:) noprint out=_moiClasses;
	run;

	proc sql noprint;
	    select name
	    into :Categories separated by ' '	
	    from _moiClasses;

	    select name    
	    into :Categories2 separated by ','
	    from _moiClasses;

	    select count(*)
	    into :NbCategories
	    from _moiClasses;
	quit;
	%put &NbCategories. &Categories. &Categories2.;

	*Get the categories strata for which frequencies will be calculated;
	%macro wrapper;
	    %do i = 1 %to &NbCategories.;
	    	%global Category&i.;
	    	proc sql noprint;
	    	    select name
	    	    into :Category&i. separated by ' '	
	    	    from _moiClasses(obs=&i.);
	    	quit;
	    %end;
	%mend;
	%wrapper;


	*Prepare the squaring matrices for the output tables;
	data _null_;
		call symputx('NUMAGECAT',countw("&AGESTRAT."," "));
		call symputx('YEARFROM',year(&startfollowup.));
		call symputx('YEARTO',year(&enddate.));
	run;
	%put &NUMAGECAT.  &YEARFROM.  &YEARTO.;

	%macro CreateSquares;

	    proc datasets nolist nowarn library=work;
	        delete _square:;
	    quit;

	    *Squaring matrix for characteristics;
	    data _squareChar(drop=i);
	    format Category $60. Characteristic $30.;

	    Category="Total unique women with pregnancy episode";
	    Characteristic="Patients";
	    output;

	    Category="Total unique pregnancy episodes";
	    Characteristic="Episodes";
	    output;

	    do i=&YEARFROM. to &YEARTO.;
	    	Category="Calendar Year of delivery admission for pregnancy episode";
	    	Characteristic=strip(put(i,10.));
	    	output;
	    end;

	    do i=1 to &NUMAGECAT.;
	    	Category="Maternal age at delivery, years(for pregnancy episode)";
	    	Characteristic=compress(scan("&AGESTRAT.",i, ' '),'DWMQY');
	    	output;
	    end;	
	    run;
		    	
	    *Treatment group matrix (has pre/post term codes);
		proc sort nodupkey data=infolder.&PREGCODES.(where=(Preterm=1 or Postterm=1)) out= _prePostTerm(keep=CodeCat Codetype Code Preterm PostTerm);
	    by Code;
	    run;

		data _prePostTerm;
		set _prePostTerm;
		Code=compress(Code,'. ');
	    length=length(code);
		run;

		%ISDATA(dataset=_prePostTerm);
		%IF %EVAL(&NOBS.>=1) %THEN %DO;
		    %MS_ProcessWildcards(InFile=_prePostTerm, 
		                         CodeVar=code, 
		                         OutFile=_prePostTerm);
		%END;

		data _preTerm (keep=code)
			 _postTerm (keep=code);
		format code $11.;
		length code $11.;
		set _prePostTerm;	
		if PreTerm=1 then output _preTerm;
		if PostTerm=1 then output _postTerm;
		run;

	    proc sql noprint;
	    	insert into _preterm values("AnyPre");
	    	insert into _postterm values("AnyPost");
	    quit;

	    data _squareCharTreat;
	    set _squareChar(in=a) _preterm(in=b) _postterm(in=c);
	    format Order best.;
	    if b or c then do;
	    	Code=compress(Code,'.');
	    	Characteristic=Code;
	    end;
	    if b then Category="Specific codes for preterm birth(for pregnancy episode)";
	    if c then Category="Specific codes for postterm birth(for pregnancy episode)";	  
	    Order=_N_;
	    drop Code ;
	    run;

	   *Control group matrix (does not have pre/post term codes);
	    data _squareCharCtrl;
	    set _squareChar(in=a);
	    format Order best.;				    	
	    Order=_N_;		    	
	    run;

	    *Squaring matrices for other tables. 
	     Must iterate backward to make sure all variables are present following the append;		
	    %do k = &NbCategories. %to 1 %by -1;	
	    	*Categories;	
	    	proc sort nodupkey data=infolder.&MOICODES.(keep=&&Category&k.) out=_temp;
	    	by &&Category&k.;
	    	run;

	    	proc append nowarn base = _squaremoi data = _temp force;
	    	run;

	    	*Delivery Years;
	    	data _temp2(drop=i);
	    	set _temp;
	    	Count=0;
	    	do i=&YEARFROM. to &YEARTO.;	
	    		DelYear=i;
	    		output;
	    	end;
	    	run;
		
	    	proc append nowarn base = _squareDelYear data = _temp2 force;
	    	run;

	    	*Age Groups;
	    	data _temp2(drop=i);
	    	set _temp;
	    	Count=0;
	    	do i=1 to &NUMAGECAT.;
	    		AgeGroup=compress(scan("&AGESTRAT.",i, ' '),'DWMQY');
	    		output;
	    	end;	
	    	run;

	    	proc append nowarn base = _squareAgeGroup data = _temp2 force;
	    	run;
	    %end;

	    proc sort nodupkey data=_squareMoi;
	    by &Categories.;
	    run;

	    proc sort nodupkey data=_squareDelYear;
	    by &Categories. DelYear;
	    run;

	    proc sort nodupkey data=_squareAgeGroup;
	    by &Categories. AgeGroup;
	    run;

	%mend CreateSquares;
	%CreateSquares;


	%macro CreateTables(episfile=, samplegroup=, condition=, analysis=);

		proc sql noprint;
		create table _ConcEpisodes as 
		select Prim.*, 
			   Sec.IndexDt as IndexDt2,
			   Sec.EpisodeEndDt as EpisodeEndDt2,			   
			   Sec.Category,
			   &Categories2.
		from &episfile. (where=(&condition.)) as Prim,
		  	 DPLocal.&RUNID._sec (where=(&condition.)) as Sec
		where Prim.PatId=Sec.PatId and Prim.Group=Sec.Group;
		quit;
			
		data _ConcEpisodes;
		set _ConcEpisodes;
		Use90=0;Any1=0;Any2=0;Any3=0;Any=0;Only1=0;Only2=0;Only3=0;All3=0;	
		
		* There is an issue in the task order Pregnancy code when the MOI exposure overlaps the 
		  whole pregnancy period. It was not being considered but should have been. This has been fixed in CIDA;
		if 0<IndexDt-IndexDt2<=90 or 0<IndexDt-EpisodeEndDt2<=90 or 
		   (IndexDt2<IndexDt-90 and EpisodeEndDt2>IndexDt) then Use90=1;
		if IndexDt<=IndexDt2<=IndexDt+90 or IndexDt<=EpisodeEndDt2<=IndexDt+90 or 
		   (IndexDt2<IndexDt and EpisodeEndDt2>IndexDt+90) then Any1=1;
		if IndexDt+91<=IndexDt2<=IndexDt+180 or IndexDt+91<=EpisodeEndDt2<=IndexDt+180 or 
		   (IndexDt2<IndexDt+91 and EpisodeEndDt2>IndexDt+180) then Any2=1;
		if IndexDt+181<=IndexDt2<=EpisodeEndDt or IndexDt+181<=EpisodeEndDt2<=EpisodeEndDt or 
		   (IndexDt2<IndexDt+181 and EpisodeEndDt2>EpisodeEndDt) then Any3=1;
		if Any1 or Any2 or Any3 then Any=1;
		if IndexDt<=EpisodeEndDt2<=IndexDt+90 then Only1=1;
		if IndexDt+91<=IndexDt2 and EpisodeEndDt2<=IndexDt+180 then Only2=1;
		if IndexDt+181<=IndexDt2<=EpisodeEndDt then Only3=1;
		if Any1 and Any2 and Any3 then All3=1;	

		* Apply the washout criterion for the Trimesters Only statistics;
		* If ConcWashPer is missing, the exposure to the secondary cohort prior to the beginning 
	      of the pregnancy episode will be ignored when assessing the trimesters only statistics;
		TrimOnlyWashMet=1;
		if not missing(ConcWashPer) then do;
			if IndexDt > EpisodeEndDt2 and IndexDt - EpisodeEndDt2 -1 < ConcWashPer then TrimOnlyWashMet=0;
		end; 

		* Do not keep secondary cohort episodes beginning after the end of primary cohort episodes;
		if IndexDt2 > EpisodeEndDt then delete;
		run;

	    *Get characteristics;
	    proc sql noprint;
	    	/*Total number of patients*/
	    	create table _NumPatients as 
	    	select "Patients" as Characteristic format=$30. length=30, count(distinct PatId) as Count 
	    	from &episfile.(where=(&condition.));

	    	/*Total number of episodes*/
	    	create table _NumEpisodes as 
	    	select "Episodes" as Characteristic format=$30. length=30, count(DelNum) as Count 
	    	from &episfile.(where=(&condition.));
	    quit;

	    *Macro that counts the number of episodes for a specific strata;
	    %macro CountbyStrata(infile=,class=,where=,out=);
	    	proc sql noprint;
	    	select count(&class.)
	    	into :NbObs
	    	from &infile.(where=(&condition.))		
	    	where &where.;
	    	quit;

	    	%IF %EVAL(&NbObs.>0) %THEN %DO;
	    		proc means data=&infile.(where=(&condition.)) noprint;
	    		var DelNum;
	    		class &class.;
	    		where &where.;
	    		output out=_By&out. (drop=_FREQ_) N=Count;
	    		run;
	    	%END;
	    	%ELSE %DO;
	    		data _By&out.;
	    		set &infile. (keep=&class. obs=0);
	    		_TYPE_=1;	    		
	    		run;
	    	%END;
	    %mend CountbyStrata;

	    %CountbyStrata(infile=&episfile.,class=DelYear,where=1,out=Year);					*By Delivery Year;
	    %CountbyStrata(infile=&episfile.,class=AgeGroup,where=1,out=AgeGroup);				*By Age Group;

	    %IF %STR("&samplegroup.")=%STR("treat") %THEN %DO;									*By Preterm and PostTerm;
		    %CountbyStrata(infile=&episfile.,class=Code,where=HadPreterm,out=PretermAny);	
		    %CountbyStrata(infile=DPLocal.&RUNID._prePostCodes,class=Code,where=Preterm,out=Preterm);
		    %CountbyStrata(infile=&episfile.,class=Code,where=HadPostterm,out=PosttermAny);
		    %CountbyStrata(infile=DPLocal.&RUNID._prePostCodes,class=Code,where=Postterm,out=Postterm);
	    %END;   

	    %IF %STR("&samplegroup.")=%STR("treat") %THEN %DO;	*Treatment group (has pre/post term codes);
	    	data _Characteristics;
	    	set _NumPatients
	    		_NumEpisodes
	    		_ByYear (in=a where=(_TYPE_=1))
	    		_ByAgeGroup (in=b where=(_TYPE_=1))
	    		_ByPreterm (in=c where=(_TYPE_=1))
	    		_ByPretermAny (in=d where=(_TYPE_=0))
	    		_BypostTerm (in=e where=(_TYPE_=1))
	    		_BypostTermAny (in=f where=(_TYPE_=0));    	
	    	format Characteristic $30.;
	    	if a then Characteristic=strip(put(DelYear,10.));
	    	if b then Characteristic=AgeGroup;
	    	if c or e then Characteristic=Code;	
	    	if d then Characteristic="AnyPre";
	        if f then Characteristic="AnyPost";	
	    	keep Characteristic Count;
	    	run;
	    %END;
	    %ELSE %DO;	*Control group (does not have pre/post term codes);
	    	
	    	data _Characteristics;
	    	set _NumPatients
	    		_NumEpisodes
	    		_ByYear (in=a where=(_TYPE_=1))
	      		_ByAgeGroup (in=b where=(_TYPE_=1));
	    	format Characteristic $30.;
	    	if a then Characteristic=strip(put(DelYear,10.));
	    	if b then Characteristic=AgeGroup;		
	    	keep Characteristic Count;
	    	run;
	    %END;

	    data _Characteristics;
	    set _Characteristics;
	    *The total number of episodes will be the denominator used for frequencies. Will be needed for aggregation later;
	    if _N_=1 then set _NumEpisodes(keep=Count rename=Count=Denom);
	    run;

	    *Merge to have a squared table;
	    proc sort nodupkey data=_squareChar&samplegroup.;
	    by Characteristic;
	    run;

	    proc sort data=_Characteristics;
	    by Characteristic;
	    run;

	    data _Characteristics;
	    merge _squareChar&samplegroup. _Characteristics;
	    by Characteristic;
	    format Analysis $15.;
	    Analysis=&analysis.;
	    if Order=1 then Denom=Count;
		if Characteristic ne "";
	    run;

	    proc sort data=_Characteristics out=_Characteristics(drop=Order);
	    by Order;
	    run;

	    proc append base = MSOC.&RUNID._chars_&samplegroup.
	                data = _Characteristics force;
	    run;


		%macro MedicationUse(where=,subanalysis=);

		    /*************************************
		     Medication Use by Trimester
		    *************************************/
	    	*For statistic involving the third trimester, we must count only the episodes that really have 3 trimesters; 	
	    	proc sql noprint;
	    	create table _Denom as 
	    	select "00_TOTAL" as Category1 format=$40. length=40,
	    		    count(DelNum) as Use90,
	    			count(DelNum) as Any1,
	    			count(DelNum) as Any2,
	    			max(sum(Has3Trim),0) as Any3,
	    			count(DelNum) as Any,
	    			count(DelNum) as Only1,
	    			count(DelNum) as Only2,
	    			max(sum(Has3Trim),0) as Only3,
	    			max(sum(Has3Trim),0) as All3,
					count(DelNum) as sumUse90,
	    			count(DelNum) as sumAny1,
	    			count(DelNum) as sumAny2,
	    			max(sum(Has3Trim),0) as sumAny3,
	    			count(DelNum) as sumAny,
	    			count(DelNum) as sumOnly1,
	    			count(DelNum) as sumOnly2,
	    			max(sum(Has3Trim),0) as sumOnly3,
	    			max(sum(Has3Trim),0) as sumAll3
	    	from &episfile.(where=(&condition.))
	    	where &where.;
	    	quit;

			/* Sum of dispensings are new statistics */
	    	%macro MedicationCount(grouping=,catnum=,out=);
	    		proc means data=_ConcEpisodes(where=(&condition.)) nway noprint missing;
	    		var Use90 Any1 Any2 Any3 Any Only1 Only2 Only3 All3;
	    		class PatId DelNum &grouping.;
	    		where &where. and strip(Category)="&catnum.";
	    		output out=_MOIby&out. (drop=_:) max(Use90 Any1 Any2 Any3 Any All3 Only1 Only2 Only3)=
											     min(TrimOnlyWashMet)=
												 sum(Use90)=sumUse90 sum(Any1)=sumAny1 sum(Any2)=sumAny2
												 sum(Any3)=sumAny3 sum(Any)=sumAny sum(All3)=sumAll3
												 sum(Only1)=sumOnly1 sum(Only2)=sumOnly2 sum(Only3)=sumOnly3;
	    		run;

	    		data _MOIby&out.;
	    		set _MOIby&out.;

				/* Because of this new criterion, the Trimeters Only stats cannot be matched 
					  with the current pregnancy program unless ConcWashPer is very high*/
				* Trimesters Only criterion;			
				if TrimOnlyWashMet=0 or Any2 or Any3 then Only1=0;
				if TrimOnlyWashMet=0 or Any1 or Any3 then Only2=0;
				if TrimOnlyWashMet=0 or Any1 or Any2 then Only3=0;
	    		
				if Any1 and Any2 and Any3 then do;
					All3=1;
					sumAll3=sumAny;
				end;

				if Only1=0 then sumOnly1=0;
				if Only2=0 then sumOnly2=0;
				if Only3=0 then sumOnly3=0;
	    		run;

	    		proc means data=_MOIby&out. nway noprint missing;
	    		var Use90 Any1 Any2 Any3 Any Only1 Only2 Only3 All3
					sumUse90 sumAny1 sumAny2 sumAny3 sumAny sumAll3 sumOnly1 sumOnly2 sumOnly3;
	    		class &grouping.;		
	    		output out=_MOIby&out. (drop=_:) sum=;
	    		run;
	    	%mend MedicationCount;

	    	*Calculate stats for each category level;
	    	%macro wrapper;
	    		proc datasets NOLIST NOWARN library=WORK;
	    		    	delete _MedicationUseTrimester;
	    		quit;

	    		*Must iterate backward to ensure all variables will be present following the append;
	    		%do k = &NbCategories. %to 1 %by -1;				
	    			%MedicationCount(grouping=&&Category&k.,catnum=Category&k.,out=Category_&k.);
	    			proc append nowarn base = _MedicationUseTrimester
	                    			   data = _moiByCategory_&k. force;
	    			run;				
	    		%end;
	    	%mend;
	    	%wrapper;
		
	    	*Square the table;
	    	proc sort data=_MedicationUseTrimester;
	    	by &Categories.;
	    	run;

	    	proc sort nodupkey data=_MedicationUseTrimester;
	    	by _ALL_;
	    	run;

	    	data _MedicationUseTrimester;
	    	merge _squaremoi _MedicationUseTrimester (in=b);
	    	by &Categories.;
	    	if not b then do;
	    		Use90=0;Any1=0;Any2=0;Any3=0;Any=0;Only1=0;Only2=0;Only3=0;All3=0;
				sumUse90=0;sumAny1=0;sumAny2=0;sumAny3=0;sumAny=0;sumAll3=0;sumOnly1=0;sumOnly2=0;sumOnly3=0;
	    	end;
	    	run;

	    	data _MedicationUseTrimester;
	    	retain &Categories. Use90 Any1 Any2 Any3 Any Only1 Only2 Only3 All3
				   sumUse90 sumAny1 sumAny2 sumAny3 sumAny sumAll3 sumOnly1 sumOnly2 sumOnly3;
	    	set _Denom _MedicationUseTrimester;
	    	format Analysis SubAnalysis $15.;
	    	Analysis=&analysis.;
	    	SubAnalysis=&subanalysis.;
	    	run;		

	    	proc append nowarn base = MSOC.&RUNID._meduse1_&samplegroup.
	                    	   data = _MedicationUseTrimester force;
	    	run;

		    /*************************************
		     Medication Use by Year
		    *************************************/	
	    	%macro MedicationGrouping(grouping=,);	

	    		proc sql noprint;
	    		create table _SquareDenom as 
	    		select distinct "00_TOTAL" as Category1 format=$40. length=70, &grouping., 0 as Count
	    		from _square&grouping.
	    		order by &grouping.;
	    		quit;

	    		proc sql noprint;
	    		create table _Denom as 
	    		select "00_TOTAL" as Category1 format=$40. length=70, &grouping., count(DelNum) as Count
	    		from &episfile.(where=(&condition.))
	    		where &where.
	    		group by &grouping.
	    	    order by &grouping.;
	    		quit;

	    		data _Denom;
	    		merge _SquareDenom _Denom;
	    		by &grouping.;
	    		run;

	    		proc transpose data=_Denom out=_Denom(drop=_NAME_) prefix=_;
	    		by Category1;
	    		id &grouping.;
	    		run;

	    		*Calculate stats for each Category level;
	    		%macro wrapper;
	    			proc datasets NOLIST NOWARN library=WORK;
	    			    	delete _MedicationUse&grouping.;
	    			quit;

	    			*Must iterate backward to ensure all variables will be present following the append;
	    			%do k = &NbCategories. %to 1 %by -1;				
	    				%MedicationCount(grouping=&&Category&k. &grouping.,catnum=Category&k.,out=Category_&k.);
	    				proc append nowarn base = _MedicationUse&grouping.
	    	                			   data = _moiByCategory_&k. force;
	    				run;				
	    			%end;
	    		%mend;
	    		%wrapper;
		
	    		*Square the table;
	    		proc sort data=_MedicationUse&grouping.;
	    		by &Categories.;
	    		run;

	    		proc sort nodupkey data=_MedicationUse&grouping.;
	    		by _ALL_;
	    		run;

	    		data _MedicationUse&grouping.;
	    		merge _square&grouping. _MedicationUse&grouping. (in=b);
	    		by &Categories. &grouping.;
	    		if b then Count=Any;
	    		keep &Categories. &grouping. Count;
	    		run;

	    		proc transpose data=_MedicationUse&grouping. out=_MedicationUse&grouping.(drop=_NAME_) prefix=_;
	    		by &Categories.;
	    		id &grouping.;
	    		run;

	    		data _MedicationUse&grouping.;
	    		retain &Categories.;
	    		set _Denom _MedicationUse&grouping.;
	    		format Analysis SubAnalysis $15.;
	    		Analysis=&analysis.;
	    		SubAnalysis=&subanalysis.;
	    		run;			

	    	%mend MedicationGrouping;
	    	%MedicationGrouping(grouping=DelYear);

	    	proc append base = MSOC.&RUNID._meduse2_&samplegroup.
	                    data = _MedicationUseDelYear force;
	    	run;

		    /*************************************
		     Medication Use by AgeGroup
		    *************************************/
	    	%MedicationGrouping(grouping=AgeGroup);

	    	proc append base = MSOC.&RUNID._meduse3_&samplegroup.
	                    data = _MedicationUseAgeGroup force;
	    	run;

		%mend MedicationUse;

	    %MedicationUse(where=1, subanalysis="ALL");
	    %MedicationUse(where=HadPreterm=1, subanalysis="PRETERM");
	    %MedicationUse(where=HadPostTerm=1, subanalysis="POSTTERM");
	    %MedicationUse(where=HadNone=1, subanalysis="NONE");


	%mend CreateTables;	

	* Loop through different groups;
	%macro tablesloop;		
		%do Group=1 %to &Ngroup.;		
			data _null_;
	       	set infolder.&COHORTFILE.;
	       	if _n_ = &Group. then do;
	            call symputx('ITGROUP',strip(Group)); 
			end;
			run;
			
			* Treatment group tables;
			%CreateTables(episfile=DPLocal.&RUNID._mstr, samplegroup=treat, condition=Group="&ITGROUP.", analysis="&ITGROUP.");

			* Control group tables;
			%createTables(episfile=DPLocal.&RUNID._ctrl, samplegroup=ctrl, condition=Group="&ITGROUP.", analysis="&ITGROUP.");

	     	* Output Table Pregnancy Episodes by Specific Enrollment Criteria;
		    proc sql noprint;
		          create table _NumPatients as 
		          select "Patients" as Characteristic format=$30. length=30, count(distinct PatId) as Count, . as EligDays, 1 as Order
		          from DPLocal.&RUNID._allDeliveries
				  (where=(Group="&ITGROUP."));

		          create table _NumEpisodes as 
		          select "Episodes" as Characteristic format=$30. length=30, count(DelNum) as Count, . as EligDays, 2 as Order
		          from DPLocal.&RUNID._allDeliveries
					(where=(Group="&ITGROUP."));

		          select Count into :DENOM from _NumEpisodes;
		    quit;

		    proc sql noprint;
		          create table _EnrolDist as 
		          select "Episodes" as Characteristic format=$30. length=30, count(DelNum) as Count, EligDays, 3 as Order 
		          from DPLocal.&RUNID._allDeliveries
					(where=(Group="&ITGROUP."))
		          group by EligDays
		          order by EligDays;
		    quit;

		    data  _EpisodesByEnrol;
		    set _NumPatients(in=a)
		        _NumEpisodes
		        _EnrolDist;
				format group $15.;
		    Denom=&Denom.;
		    if a then Denom=Count;
			group="&ITGROUP.";
		    run;

	    	proc append base = MSOC.&RUNID._EpisodesByEnrol
	                    data = _EpisodesByEnrol force;
	    	run;

		%end;
	%mend tablesloop;
	%tablesloop;

	%put NOTE: ******** END OF MACRO: ms_cidatablest4 v1.0 ********;

%mend ms_cidatablest4;
