****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_createcontrolgroup.sas  
*
* Created (mm/dd/yyyy): 08/24/2015
* Last modified: 05/25/2016
* Version: 2.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will create control groups.                                        
*   
*  Program inputs:  
*	- a dataset containing enrolment episodes
*	- a dataset containing inclusion/exclusion codes
*	- a dataset containing defined group pregnancy episodes
*	- a dataset containing the patient master list
*             	                        
*  Program outputs:                                                                                                                                       
*	- datasets with a list of control patients and demo data
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*    2.0	  25MAY2016  SL			Implement enhanced HOI
***************************************************************************************************;



%macro ms_CreateControlGroup();

	%put =====> MACRO CALLED: ms_CreateControlGroup v2.0;

	* Create basic control population;
	%ms_agestrat(infile=ENR_&GROUP.,
	             outfile=_ControlPop,
	             startdt=birth_date,
	             enddt=Enr_End,
	             timestrat=&agestrat.);

	* Apply age group and enrollment duration requirements;
	data _ControlPop (where=(Overlap=1 and Enr_end-Enr_start >= &EnrDays.));
	set _ControlPop;
	Overlap= %MS_PeriodsOverlap(period1=Enr_Start Enr_End,period2=MinAgeDate MaxAgeDate);
	run; 


	* Identify enrollment episodes where members also appeared in the treatment group;
	proc sort data= _ControlPop;
	by PatID Enr_Start Enr_End;
	run;

	proc sort nodupkey data= _GroupEpis out=_UniqueGroupEpis;
	by patid Episode;
	run;

	proc sort data= _UniqueGroupEpis out=_UniqueGroupEpis;
	by PatID Enr_Start Enr_End;
	run;

	data _ControlPopEpisTreat _ControlPopEpisNoTreat;

	merge _ControlPop (in=a) _UniqueGroupEpis;
	by PatID Enr_Start Enr_End;
	if a;
	if Enr_Start <= ADate <= Enr_End then output _ControlPopEpisTreat;	*Enrollment episodes where treatment occured;
	else output _ControlPopEpisNoTreat;									*Enrollment episodes without treatment;
	keep PatID Enr_Start Enr_End Birth_Date;
	run;

	data _UniqueGroupEpis;	
	set _UniqueGroupEpis;	
	drop Enr_Start Enr_End;	*Must be removed before shave inside macro call below;
	run;

	*Format dates to avoid w.a.r.n.i.n.g - used to be MMDDYY10.;
	data _ControlPopEpisTreat;
	set _ControlPopEpisTreat;
	format enr_start enr_end date9.;
	run;

	* Remove treatment episodes segments from the control population enrollment episodes;
	%ms_shaveinside(reffile=_ControlPopEpisTreat,
		             refstart=Enr_Start,
		             refend=Enr_End,                         
		             ToShaveFile=_UniqueGroupEpis,
		             ToShaveStart=EpisodeStartDt,
		             ToShaveEnd=EpisodeEndDt,
		             outfile=_ControlPopEpisTreat);

	* Get birth_date back;
	proc sql noprint  undo_policy=none;
	create table _ControlPopEpisTreat as
	select cont1.*,
		   cont2.birth_date
	from _ControlPopEpisTreat as cont1, _ControlPop as cont2
	where cont1.patid=cont2.patid;
	quit;
		
	data _ControlPopulation;
	set _ControlPopEpisTreat 
	    _ControlPopEpisNoTreat;
	run;

	proc sort nodupkey data= _ControlPopulation;
	by PatID Enr_Start Enr_End;
	where Enr_end-Enr_start >= &enrdays.; *Reapply the criterion since removing treatment episodes may have created too short segments;
	run;

	/* There are some discrepancies with the original pregnancy code caused by the fact that the shaveinside macro does 
	* not only shave the enrollment episode where the delivery occurs but shaves also previous enrollment episodes
	* For example, enrollment episode 1 (01AUG2004 - 31MAY2006) and episode 2 (01AUG2006 - 31JAN2007), delivery 19NOV2006
	* ShaveInside will truncate episode 1 at 19FEB2006 even if the delivery occured in episode 2
	*/ 
	data _ControlPopulation;
	set _ControlPopulation;
	idc = _N_;
	MinAge = floor((intck('month',Birth_Date,Enr_Start+&enrdays.) - (day(Enr_Start+&enrdays.) < day(Birth_Date))) / 12);
	MaxAge = floor((intck('month',Birth_Date,Enr_End) - (day(Enr_End) < day(Birth_Date))) / 12);
	*Dummies used when there is no PEC;
	CStart=-99999;
	CEnd=999999;*If no inclusion exclusion, leave as is;
	run;


*START code for complex HOI; 

	*Create a copy to have control info in the case of exclusion only;
	data _ControlPopulation_copy;
	set _ControlPopulation;
	run;

	* Extract highest CONDFROM value among exclusions if any: apply if GROUP has incl-excl codes, else do not change _controlpopulation before matching;
    %let MaxExclCondFrom=0;
    %macro wrapper;
    	%ISDATA(dataset=&INCLUSIONCODES.);
    	%IF %EVAL(&NOBS.>=1) %THEN %DO;
			data _IT&INCLUSIONCODES.;
			set &INCLUSIONCODES.;
			where group="&ITGROUP.";
			run; 

	    	%ISDATA(dataset=_IT&INCLUSIONCODES.);
			%IF %EVAL(&NOBS.>=1) %THEN %DO;
	    		proc sql noprint;
	    		select min(condfrom) into :MaxExclCondFrom 						
	    		from &INCLUSIONCODES.(where=(group="&ITGROUP.")) 
	    		where Condinclusion=0;
	    		quit;

			    data _ControlPopulation;
			    set _ControlPopulation;
			    format  Enr_Start Enr_End DenomEnrStart DenomEnrEnd mmddyy10.;

				*Note: in pregnancy, enrdays is from delivery date, not episode start date and it is already applied above;	
			    MaxLookBack=Max(0,-&MaxExclCondFrom.);	

			    * Resctrict eligibility to Study period (minus maximum of required lookbacks);
			    DenomEnrStart=Enr_Start;
			    * Initially truncate start eligibility enrollment start by max of lookback periods;
			    DenomEnrStart=DenomEnrStart + MaxLookBack; 									
			    DenomEnrEnd=Enr_End;
			    if DenomEnrEnd >= DenomEnrStart;

				*Create MaxLookBack macro variable;
				call symputx('MaxLookBack',MaxLookBack);

				*If there is inclusion or exclusion use DenomEnr instead of CStart CEnd;
				drop CStart CEnd;
			    run;
				%put &MaxLookBack.;

		    %END;
		    %put &MaxExclCondFrom.;

		%END;
	   %mend wrapper;
    %wrapper;


	/*---------------------------------------------------*/
	/* Inclusion required (POV3 Incl) for eligibility	 */
	/*---------------------------------------------------*/  
	proc datasets library=work nowarn nolist;
	delete _control_stack:;
	quit;

	%macro wrapper;
		%ISDATA(dataset=&INCLUSIONCODES.);
		%IF %EVAL(&NOBS.>=1) %THEN %DO;

			data _IT&INCLUSIONCODES.;
			set &INCLUSIONCODES.;
			where group="&ITGROUP.";
			run; 

			*For EACH condition, shave periods for all subconditions;
			%ISDATA(dataset=_IT&INCLUSIONCODES.);
			%IF %EVAL(&NOBS.>=1) %THEN %DO;
				%let MAXCOND=;
				proc sql;
				select max(cond) into: maxcond
				from _IT&INCLUSIONCODES.;
				quit;
				%put &MAXCOND.;

				%DO COND=1 %TO &MAXCOND.;
					*Determine how many levels of subconditions needed;
					%let MAXSUBCOND=;
					proc sql;
					select max(subcond) into: maxsubcond
					from _IT&INCLUSIONCODES. (where=(COND=&COND.));
					quit;
					*Determine if condition is an inclusion;
					proc sql;
					select max(CondInclusion) into: inclusion
					from _IT&INCLUSIONCODES. (where=(COND=&COND.));
					quit;
					%put &MAXSUBCOND. &INCLUSION.;

					*Determine if there is subexclusion in condition in order to calculate new maxlookback since one above is for CondInclusion=0;
					%let SUBEXCL=0;
		            data _null_;
		            set _IT&INCLUSIONCODES.(where=(COND=&COND.));
		            if subCondinclusion=0 then call symputx("SUBEXCL",1);
		            run;
		            %put SUBEXCL = &SUBEXCL.;

					*If there is subexclusion, determine new maxlookback for that specific condition before shaving;
					%IF &SUBEXCL.=1 %THEN %DO;
					    %let NewMaxExclCondFrom=0;
					    proc sql noprint;
					    select min(condfrom) into :NewMaxExclCondFrom 
					    from _IT&INCLUSIONCODES. 									
					    where COND=&COND. and SubCondInclusion=0;
					    quit;
					    %put &NewMaxExclCondFrom.;

					    data _ControlPopulation_&cond. (rename=(newmaxlookback=MaxLookBack));
					    set _ControlPopulation;
						*If the subcond condfrom is greater in absolute value than the maxlookback value, then readjust denom period;
					    NewMaxLookBack=Max(0,&Maxlookback.,-&NewMaxExclCondFrom.);
					    * Resctrict eligibility to Study period (minus maximum of required lookbacks);
					    DenomEnrStart=Enr_Start;
					    * Initially truncate start eligibility enrollment start by max of lookback periods;
					    DenomEnrStart=DenomEnrStart + NewMaxLookBack;
					    DenomEnrEnd=Enr_End;
					    if DenomEnrEnd >= DenomEnrStart;
						  drop maxlookback;
					    run;
					%END;*If subexcl=1;
					%IF &SUBEXCL.^=1 %THEN %DO;
						data _ControlPopulation_&cond.;
						set _ControlPopulation;
						run;
					%END;

					%DO SUBCOND=1 %TO &MAXSUBCOND.;
						*Determine if subcond is an inclusion or exclusion;
						%let SUBINCLUSION=0;
	 					proc sql;
						select max(SubCondInclusion) into: subinclusion
						from _IT&INCLUSIONCODES. (where=(COND=&COND. and SUBCOND=&SUBCOND.));
						quit;
						%put &SUBINCLUSION.;

						*Merge time periods for each subcondition and shave for each;
						%IF &SUBINCLUSION. = 1 %THEN %DO;
							  data _EligIncl_&subcond.;
						    set _InclExcl;
						    where COND=&COND. and subcond=&SUBCOND.;
						    format EligStart EligEnd date9.;
						    EligStart=ADate-coalesce(CondTo,99999);
						    EligEnd=Expiredt-coalesce(CondFrom,-99999);
						    if EligStart<=EligEnd;
						    keep PatId ADate EligStart EligEnd;
						    run;

							%MergeTimePeriods(dataset=_EligIncl_&subcond.,
							                  DateStart=EligStart,
							                  DateEnd=EligEnd);	

							%ms_shaveoutside(reffile=_EligIncl_&subcond.,
							              	 refstart=EligStart,
							               	 refend=EligEnd,  
										   	       KeepPartBf=Y,
							               	 ToShaveFile=_controlpopulation_&cond.,
							               	 ToShaveStart=DenomEnrStart,
							               	 ToShaveEnd=DenomEnrEnd,
							               	 outfile=_controlpopulation_&cond.);

							*Drop refstart and refend since _Denom&Group can be looped;
							data _controlpopulation_&cond.;
							set _controlpopulation_&cond.(drop=eligstart eligend);
							run;
						%END;*If subinclusion=1;
						%IF &SUBINCLUSION. = 0 %THEN %DO;
					        data _UneligExcl_&subcond.;
					        set _InclExcl;
						      where COND=&COND. and subcond=&SUBCOND.;
					        format UneligStart UneligEnd MMDDYY10.;
					        UneligStart=ADate-coalesce(CondTo,99999);
					        UneligEnd=Expiredt-coalesce(CondFrom,-99999);
					        if UneligStart<=UneligEnd;
					        keep PatId ADate UneligStart UneligEnd;
					        run;

							%MergeTimePeriods(dataset=_UneligExcl_&subcond.,
							                  DateStart=UneligStart,
							                  DateEnd=UneligEnd);

							*Verify if there is data or else e.r.r.o.r when shaving;
							%ISDATA(dataset=_UneligExcl_&subcond.);
							%IF %EVAL(&NOBS.>=1) %THEN %DO;
							    %ms_shaveinside(reffile=_controlpopulation_&cond.,
							                   	refstart=DenomEnrStart,
							                    refend=DenomEnrEnd,  
							                    ToShaveFile=_UneligExcl_&subcond.,
							                    ToShaveStart=UneligStart,
							                    ToShaveEnd=UneligEnd,
							                    outfile=_controlpopulation_&cond.);
							%END;*If there is data;
						%END;*If subinclusion=0;
					%END;*loop on subcond;

					%put &INCLUSION.;
					%IF &INCLUSION.=1 %THEN %DO;
						proc append base=_control_STACK_INC
							data=_controlpopulation_&cond.(keep=patid denomenrstart denomenrend) force;
							run;
					%END;

					%IF &INCLUSION.=0 %THEN %DO;
						proc append base=_control_STACK_EXCL
							data=_controlpopulation_&cond.(keep=patid denomenrstart denomenrend) force;
						    run;
					%END;

				%END;*End Loop Cond;

				*Output dataset for inclusion=1 and for exclusions=0 and then (a) shaveoutside (b) shaveinside denom; 
				*For inclusions, need to merge periods;

				*First shaveoutside denom file but need to merge inclusion periods together;
				%IF %SYSFUNC(exist(_CONTROL_STACK_INC))=1 %THEN %DO;
					%MergeTimePeriods(dataset=_CONTROL_STACK_INC,
							          DateStart=DenomEnrStart,
							          DateEnd=DenomEnrEnd);


			        *_Denom&Group._STACK_INC includes all time periods where the potential index dates will meet at least one of the cond inclusions;
					data _CONTROL_STACK_INC;
					set _CONTROL_STACK_INC;
					rename 
					DenomEnrStart=DTSTART
					DenomEnrEnd=DTEND
					;
					run;

					%ms_shaveoutside(reffile=_CONTROL_STACK_INC,
								   	 refstart=DTSTART,
							       	 refend=DTEND,  
								   	 /*id=EligEpisode,*/ 
								   	 KeepPartBf=Y,
							       	 ToShaveFile=_controlpopulation,
							       	 ToShaveStart=DenomEnrStart,
							       	 ToShaveEnd=DenomEnrEnd,
							       	 outfile=_controlpopulation);	

				%END;

				%IF %SYSFUNC(exist(_CONTROL_STACK_EXCL))=1 %THEN %DO;

			        *merge all periods of ineligibilities;
					%MergeTimePeriods(dataset=_CONTROL_STACK_EXCL,
							          DateStart=DenomEnrStart,
							          DateEnd=DenomEnrEnd);

				      *_Denom&Group._STACK_EXCL includes all time periods where the potential index dates will meet at least one of the cond exclusions;
					data _CONTROL_STACK_EXCL;
					set _CONTROL_STACK_EXCL;
					rename 
					DenomEnrStart=DTSTART
					DenomEnrEnd=DTEND
					;
					run;

					*Defensive coding if exclusion codes only;
					data _controlpopulation;
					format DenomEnrStart DenomEnrEnd Date9.;
					set _controlpopulation;
					run;

			        *remove all these periods from the ELIG;
					%ms_shaveinside(reffile=_controlpopulation,
						           	refstart=DenomEnrStart,
						            refend=DenomEnrEnd,  
									/*id=ENR_START ENR_END,*/				    
						            ToShaveFile=_CONTROL_STACK_EXCL,
						            ToShaveStart=DTSTART,
						            ToShaveEnd=DTEND,
						            outfile=_controlpopulation);
				%END;

				proc sort data=_controlpopulation out=_controlpopulation nodupkey;						
				by patid denomenrstart denomenrend;
				run;

				*Need to rename if inside the inclusion code loop;
				data _controlpopulation;
				set _controlpopulation ;
				rename 
				DenomEnrStart=CStart
				DenomEnrEnd=CEnd;
				run;

				*If there was exclusion, other variables lost in shaveinside so merge back in;
				proc contents data=_controlpopulation out=_name (where=(name="ENR_START"));
				run;
				%ISDATA(dataset=_name);
				%IF %EVAL(&NOBS.=0) %THEN %DO;
					*Match treatment with control;
					proc sql noprint;
					create table _tmp_controlpopulation as
					select cont.Patid,
						cont.CStart,
						cont.CEnd,
						info.enr_start,
						info.enr_end,
						info.birth_date,
						info.minage,
						info.maxage
			      	from _controlpopulation as cont,
					     _controlpopulation_copy as info
					where info.enr_start <= cont.Cstart<= info.enr_end and info.patid=cont.patid
					order by patid; 
					quit;

					data _controlpopulation;
					set _tmp_controlpopulation;
					run;
				%END;

			%END;*If there are inclusion codes for GROUP;
		%END;*If there are inclusion codes;

%mend wrapper;
%wrapper;


/*-----------------------------------------------------*/
/* Make sure the number of enrollment days remaining   */
/* after each potential elig days is sufficient		   */
/*-----------------------------------------------------*/
* Extract highest CONDTO value among exclusions if any;
%let MaxExclCondTo=0;
%macro wrapper;
	%ISDATA(dataset=&INCLUSIONCODES.);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;
		proc sql noprint;
		select max(condto) into :MaxExclCondTo 
	   	from &INCLUSIONCODES.(where=(group="&ITGROUP.")) 
		where condinclusion=0;
		quit;
	%END;
	%put &MaxExclCondTo.;
%mend wrapper;
%wrapper;

	proc sort data=_controlpopulation;by patid;run;

    data _controlpopulation;
	retain patid birth_date Enr_Start Enr_End CStart CEnd IDC minage maxage;
    set _controlpopulation;
    by PatId;
    format AdjustedEnrEnd date9.;

    * Adjust eligibility end date;
    AdjustedEnrEnd = Enr_End - Max(0,&MaxExclCondto.);
    CEnd = min(CEnd,AdjustedEnrEnd);
	IDC=_N_;
	drop adjustedenrend;
    
	MinAge = floor((intck('month',Birth_Date,Enr_Start+&enrdays.) - (day(Enr_Start+&enrdays.) < day(Birth_Date))) / 12);
	MaxAge = floor((intck('month',Birth_Date,Enr_End) - (day(Enr_End) < day(Birth_Date))) / 12);
	if (Enr_end-Enr_start) >= &enrdays.;
	run;

*END code for complex HOI; 

	data _TreatmentGroupForMatch (drop=adate enr_end enr_start EpisodeStartDt t4fupwashper washper);
	set _PtsMasterList;
	idt = _N_;
	age = floor((intck('month',Birth_Date,EpisodeEndDt) - (day(EpisodeEndDt) < day(Birth_Date))) / 12);
	run;

	%macro MatchPatientsForControl();

	    proc datasets library=work nolist nowarn;
	    delete _MatchingResults;        
	    quit;

	    * Get treatment minimum and maximum delivery age;
	    proc sql noprint;
	        select min(age)
	        into :MinAge	
	        from _TreatmentGroupForMatch;

	        select max(age)
	        into :MaxAge	
	        from _TreatmentGroupForMatch;
	    quit;

	    %put &MinAge. &MaxAge.;

	    * Create control datasets according to each age;
	    data 
	        %do i=&MinAge. %to &MaxAge.;
	            _control_&i.
	        %end;
	    ;
	    set _ControlPopulation;
	        %do i=&MinAge. %to &MaxAge.;
	            if MinAge <= &i. <= MaxAge then output _control_&i.;
	        %end;
	    run;

	    * Loop on each age;
	    %do i=&MinAge. %to &MaxAge.;

	        *Match treatment with control;
	        proc sql noprint;
	            create table _MatchingResults_&i. as
	            select treat.IDT,
	                   contr.IDC,
	    	           ranuni(123457) as ToRandSelect
	            from _TreatmentGroupForMatch(keep=IndexDt IDT EpisodeEndDt Age where=(Age=&i.)) as treat,
	                 _control_&i. as contr
	            where treat.EpisodeEndDt <= contr.Enr_End and contr.Enr_Start <= treat.EpisodeEndDt-&enrdays. and /*at least x ENRDAYS before deldate*/
					  contr.CStart <= treat.IndexDt <= contr.CEnd and											  /*start pregnancy during control period*/	
	                  floor((treat.EpisodeEndDt-contr.Birth_date)/365.25) = treat.Age							  /*Age at treated deldate = treated age*/
	            order IDT, ToRandSelect; 
	        quit;

	    	* Keep one control per treatment;
	        data _MatchingResults_&i.;
	        set _MatchingResults_&i.;
	        by IDT;
	        if first.IDT;
	        run;

	    	%IF %SYSFUNC(exist(_MatchingResults_&i.))=1 %THEN %DO;
	     	    proc datasets library=work nolist;
	            append base=_MatchingResults data=_MatchingResults_&i. FORCE;        
	            quit;
	    	%END;

	    %end;

	    proc datasets library=work nolist;
	    delete _matchingresults_: _control_:;
	    quit;

	%mend MatchPatientsForControl;

	*Do no run the matching macro if _TreatmentGroupForMatch is empty but create an empty table instead;
	%ISDATA(dataset=_TreatmentGroupForMatch);
	%IF %EVAL(&NOBS.>=1) %THEN %DO;
		%MatchPatientsForControl();
	%END;
	%ELSE %DO;	
		data _matchingresults;
		format idt idc torandselect best12.;
		call missing(idt, idc, torandselect);
		if _N_ = 0 then output;
		stop;
		run;
	%END;

	proc sql noprint;
	    Create Table _ControlGroup as
	    select contr.PatId,
	           contr.Birth_Date,
	           contr.Enr_Start,   
	           contr.Enr_End,   	   
	           treat.EpisodeEndDt,
	           treat.IndexDt,
	    	   treat.Code,
	    	   treat.DelNum,		   
	    	   treat.AgeGroup,
	      	   treat.DelYear,	    	   
	    	   treat.HadPreTerm,
	    	   treat.HadPostTerm,
	    	   treat.HadNone,	    	   
	    	   treat.Has3Trim,
			   treat.ConcWashPer

	    from _ControlPopulation as contr,
	    	 _TreatmentGroupForMatch as treat,
	         _matchingresults as mres

	    where contr.idc = mres.idc and treat.idt = mres.idt;
	quit;

*Defensive coding to avoid w.a.r.n.i.n.g in cidanum if first group does not go thru incl-excl macro;
data  _ControlGroup;
format Enr_Start Enr_End DATE9.;
set _ControlGroup;
run;


	%put NOTE: ******** END OF MACRO: ms_CreateControlGroup v2.0 ********;

%mend ms_CreateControlGroup;

