****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_processInputfiles.sas  
*
* Created (mm/dd/yyyy): 12/19/2014
* Last modified: 11/30/2016
* Version: 1.9
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   This macro will process the program input files variables.                                        
*   
*  Program inputs:  
*	- a dataset containing the cohort definition codes
*	- a dataset containing inclusion/exclusion codes
*	- a dataset containing the monitoring file period
*	- a dataset containing the cohort definition parameters
*             	                        
*  Program outputs:                                                                                                                                       
*	- datasets with formatted input files variables
*   - _diag, _proc, _ndc and _labs that will be used for code extraction
*
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*     1.3     04/13/15		DM		Added defensive coding to remove true duplicates in the input files
*
*     1.4     04/21/15		EM		Created the looks dataset to have looks with non overlapping dates
*                                   Added defensive coding checks to check surveillance parameters accuracy
*
*	  1.5	  06/26/2015	DM      Added code to ensure HDPS variable integrity. 
*								    Removed the code from 1.4 related to Surveillance activity
*
*	  1.6	  09/14/2015	DM      Added code to process new Type 4 analysis input files
*
*	  1.7	  01/15/2016	VC      PreExisting conditions enhancements
*
*	  1.8	  06/27/2016	DM		Added the code removed in 1.5 related to Surveillance activity
*
*	  1.9	  11/30/2016 	AP		Added code to adjust EndDate if DP Max Date < EndDate
*
***************************************************************************************************;

%macro ms_processInputfiles();

%put =====> MACRO CALLED: ms_processInputfiles v1.9;

/**********************************************/
/* Diagnosis, Procedure, and Dispensing codes */
/**********************************************/

*Cohortcodes;	
	data &COHORTCODES.;
	set infolder.&COHORTCODES.;
    Code=compress(Code,'. ');
	run;	

*InclusionCodes;
    %IF %SYSFUNC(exist(infolder.&INCLUSIONCODES.))=1 %THEN %DO;

	    data &INCLUSIONCODES.;
	    set infolder.&INCLUSIONCODES.;

	    if CondInclusion=1 then do;
	        T1_INDEX="INC";
	        T2_INDEX="INC";
	        T3_INDEX="INC";
			T4_INDEX="INC";
	    end;
	    if CondInclusion=0 then do;
	        T1_INDEX="EXC";
	        T2_INDEX="EXC";
	        T3_INDEX="EXC";
			T4_INDEX="EXC";
	    end;

	    T2_FUP="NOT";
	    T3_COMP="NOT";	
	    T3_RISK="NOT";
		T4_FUP="NOT";

		*Defensive coding: upcase condlevel and subcondlevel;
		condlevel=upcase(condlevel);
		subcondlevel=upcase(subcondlevel);
		 
	    run;

		proc sort data=&INCLUSIONCODES. out=&INCLUSIONCODES.;
		by group CondLevel SubCondLevel;
		run;

		*Create numeric Condlevel and SubCondlevel variables for looping;
		data &INCLUSIONCODES. (drop=l_subcond l_cond);
		set &INCLUSIONCODES.;
		format cond subcond 8.;
		by group CondLevel;

		l_cond=lag(CondLevel);
		l_subcond=lag(SubCondLevel);

		if first.group then do;
			cond=1;
			call missing(l_cond);
		end;
		else do;
			if CondLevel=l_cond then do;
				cond=cond;
			end;
			else if CondLevel^=l_cond then do;
				cond=cond+1;
			end;
		end;
		*Create numeric subcondlevel variable;
		if first.CondLevel then do;
			subcond=1;
			call missing(l_subcond);
		end;
		else do;
			if SubCondLevel=l_subcond then do;
				subcond=subcond;
			end;
			else if SubCondLevel^=l_subcond then do;
				subcond=subcond+1;
			end;
		end;
		retain cond subcond;
		run;

	    *Append inclusion and Exclusion codes to work.&COHORTCODES.;
	    data &COHORTCODES.;
	    set &COHORTCODES.
	        &INCLUSIONCODES.;
	    Code=compress(Code,'. ');
	    run;
    %END;

*TYPE4 Pregnancy Pre/Post term codes;
	%IF %SYSFUNC(exist(infolder.&PREGCODES.))=1 %THEN %DO;
		data &PREGCODES.;
		set infolder.&PREGCODES.;
		Code=compress(Code,'. ');
		T1_INDEX="NOT";
        T2_INDEX="NOT";
        T3_INDEX="NOT";
		T4_INDEX="PCY";
		T2_FUP="NOT";
	    T3_COMP="NOT";	
	    T3_RISK="NOT";
		T4_FUP="NOT";
		run;

		* Make sure pre/post term codes with the same priorities have the same duration;
		proc sql noprint;	
			create table _DurationFreq as 
			select count(distinct Duration) as NumDuration, PreTerm, PostTerm, Priority
			from &PREGCODES.
			group by PreTerm, PostTerm, Priority;

			select max(NumDuration) into :NumDuration 
			from _DurationFreq;
		quit;
		
		%if (&NumDuration. gt 1) %then %do;
	    	%put WARNING: Some Preterm or Posterm have the same priority but different duration;	     
	    %end;

		data &COHORTCODES.;
	    set &COHORTCODES.
	        &PREGCODES.;	    
	    run;
	%END;

*TYPE4 Medication of interest/drug use codes;
	%IF %SYSFUNC(exist(infolder.&MOICODES.))=1 %THEN %DO;
		data &MOICODES.;
		set infolder.&MOICODES.;
		Code=compress(Code,'. ');
		T1_INDEX="NOT";
        T2_INDEX="NOT";
        T3_INDEX="NOT";
		T4_INDEX="DEF2";
		T2_FUP="NOT";
	    T3_COMP="NOT";	
	    T3_RISK="NOT";
		T4_FUP="NOT";
		run;		

		data &COHORTCODES.;
	    set &COHORTCODES.
	        &MOICODES.(in=b);	
	    run;

		*Get the total number of categories to process and create groups for episodes creation;
		proc contents data=infolder.&MOICODES.(keep=Category:) noprint out=_moiClasses;
		run;

		%macro ProcessCategories;
			proc sql noprint;
				select count(Name)
			    into :NbCategories
			    from _moiClasses;
			quit;

		    %do i = 1 %to &NbCategories.;
		    	%global Category&i.;
				%global CategorySQL&i.;
		    	proc sql noprint;
		    	    select name
		    	    into :Category&i. separated by ' '	
		    	    from _moiClasses(obs=&i.);

					select name
		    	    into :CategorySQL&i. separated by ','	
		    	    from _moiClasses(obs=&i.);
		    	quit;
		    %end;
		%mend ProcessCategories;
		%ProcessCategories;
	%END;

	*Defensive coding - remove input files true duplicates;
	proc sort nodupkey data=&COHORTCODES.;
	by _ALL_;
	run; 

/*******************/
/* Monitoring File */
/*******************/

    %IF %SYSFUNC(exist(infolder.&Monitoringfile.))=1 %THEN %DO;
        *limiting to periodid set by user;
        *Default period selection;
        data _null_;
		%IF %STR(&PERIODIDSTART.) = %STR() or %LENGTH(&PERIODIDSTART.) = 0. %THEN %DO;
			call symputx("PERIODIDSTART",1);
		%END;
		%IF %STR(&PERIODIDEND.) = %STR() or %LENGTH(&PERIODIDEND.) = 0. %THEN %DO;
			call symputx("PERIODIDEND",99999);
		%END;        
        run;

    	*Defensive coding: make sure the farthest ending date is considered;

    *	proc sort data=infolder.&Monitoringfile. out=&Monitoringfile.;
    *	by enddate;
    *   where &PERIODIDSTART. <= PeriodId  <= &PERIODIDEND.;
    *	run; 

    *Taking min startfollowup and max enddate;
        proc means data=infolder.&Monitoringfile. nway noprint;
        var startfollowup enddate ;
        where &PERIODIDSTART. <= PeriodId  <= &PERIODIDEND.;
        output out=&Monitoringfile._agg min(startfollowup)=startfollowup
                                    max(enddate)=enddate;
        run; 

		data &Monitoringfile.; 
			set infolder.&Monitoringfile.;
		run;

   		%IF %SYSFUNC(exist(infolder.&TYPE2FILE.))=1 %THEN %DO;
			data _null_;
				set infolder.&TYPE2FILE.;
				  if _n_ = 1 then do; /*Only 1 Censor_dpend value allowed*/
           			 call symputx('Censor_DPEnd',upcase(strip(Censor_DPEnd)));  
				  end;
			run;

				/*Turn off CENSOR_DPEND if using SURVEILLANCEMODE*/
				%IF "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %THEN %DO;
					%if &Censor_DPEnd. = Y %then %do;
						%let Censor_DPEnd = N;
						%put WARNING: Censor_DPEnd turned off because Surveillance mode is being used;
					%end;
				%end;

			%global dpmax_adjust;

			%if &Censor_DPEnd. = Y %then %do;

				/*add dp_maxdate to monitoring file*/
				data &Monitoringfile._agg(drop=dp_maxdate);
					set &Monitoringfile._agg;
					format dp_maxdate date9.;
					dp_maxdate = &DP_MaxDate.;

					if dp_maxdate < enddate then do;
						enddate = dp_maxdate; /*recode enddate*/
						call symputx("DPMax_Adjust", 1); /*Set DPMax_adjust flag to track adjustment*/
					end;
				run;

				data &Monitoringfile.(drop=dp_maxdate);
					set &Monitoringfile.;
					format dp_maxdate date9.;
					dp_maxdate = &DP_MaxDate.;

					if dp_maxdate < enddate then do;
						enddate = dp_maxdate; /*recode enddate*/
					end;
				run;
			%end;
		%end;

        %global startfollowup enddate numperiodid;
        data &Monitoringfile._agg;
        set &Monitoringfile._agg /*end=eof1*/;
    *   if eof1 then do;
        call symputx("startfollowup",startfollowup);
        call symputx("enddate",enddate);
     *  end;
        run;

        %put &startfollowup. &enddate. &numperiodid.;

		* Create Non Overlapping Looks dates;
        proc sort data=&Monitoringfile. out=looks;
        by PERIODID;
        run;
        data looks;
        set looks;
        format lenddate LookStart LookEnd date9.;
        LookStart=startfollowup;
        LookEnd=enddate;
        if periodid>1 then LookStart=lenddate+1;
        lenddate=enddate;
        retain lenddate;
        rename PeriodId = Look;
        keep PeriodId LookStart LookEnd;
        run; 
    %END;

	%IF "&SURVEILLANCEMODE."="f" or "&SURVEILLANCEMODE."="p" %THEN %DO;

  		%if &PERIODIDSTART. ne &PERIODIDEND. %then %do;
  			%put WARNING: Surveillance mode runs for only one period, i.e. PERIODIDSTART must be equal to PERIODIDEND.;
 	        %abort;
  		%end;

    	* Prior data existence for Surveillance;
    	%IF %EVAL(&PERIODIDSTART.>=2) %THEN %DO;

        %let DPLPrior = %ms_check_path(&DPLPrior);
        %put &DPLPrior.;

        libname DPLPrior "&DPLPrior." access=readonly;

      	%ISDATA(dataset=DPLPRIOR.&RUNID._mstr);	
        %put &NOBS.;
        %IF %EVAL(&NOBS.=0) %THEN %DO;
    			%put WARNING: DPLPRIOR.&RUNID._mstr not found or is empty.;
 	        %abort;
        %END;

        
      	%ISDATA(dataset=DPLPRIOR.&RUNID._signature);	
        %put &NOBS.;
        %IF %EVAL(&NOBS.>0) %THEN %DO;
          data _null_;
          set DPLPRIOR.&RUNID._signature;
          if var="PERIODIDEND" then call symput("LASTRUN",strip(VALUE)); 
          run;
          %put &LASTRUN.;
          %if %eval(&PERIODIDSTART.-1 ne &LASTRUN.) %then %do;
      			%put WARNING: DPLPrior is not pointing to the last run.;
            %abort;
          %end;
        %END;

        %if %str(%UPCASE("&ANALYSIS."))=%str("PS") %then %do;

        	proc sql noprint;	
      	  	  select max(comp_order) 
      	  	  into :Max_comp_order
      	  	  from infolder.&COMPARISON.;
          quit;

          %do k=1 %to &Max_comp_order.;
          	%ISDATA(dataset=DPLPRIOR.&runID._matched_&k._%eval(&PERIODIDSTART.-1));	
            %IF %EVAL(&NOBS.=0) %THEN %DO;
        			%put WARNING: DPLPRIOR.&runID._matched_&k._%eval(&PERIODIDSTART.-1)) not found or is empty.;
              %abort;
            %END;
          %end; 

        %end;

      %END;  /*PERIODIDSTART>=2*/

  	%END;

/*********************************/
/* Preprare Raw extraction files */
/*********************************/

	data _diag(drop=comb)
         _proc(drop=comb)
	     _ndc;
    set &COHORTCODES.;
    Code=compress(Code,'. ');
    length=length(code);
    CodeCat=upcase(CodeCat);
    comb=0;
    if CodeCat in('RX') then output _ndc;
    if CodeCat in('DX') then output _diag;
    if CodeCat in('PX') then output _proc;
    keep Code CodeCat Codetype comb;
    run;

    *checking for the presence of optional lab vars;
    data _null_;
    dset=open("&COHORTCODES.");
    call symput ('Labvars',put(varnum(dset,'RawLabResult'),best.));
    run;
    %put &Labvars.;

    %IF %EVAL(&Labvars.>=1) %THEN %DO;
    	data _lab;
        set &COHORTCODES.;
        Code=compress(Code,'. ');
        length=length(code);
        CodeCat=upcase(CodeCat);
        RawCaresetting=Caresettingprincipal;
        RawGroup=Group;
        if CodeCat in('LB') then output _lab;
        keep Code CodeCat Codetype RawLabResult RawLabDateType RawCaresetting RawGroup;* RawCaresetting;
        run;
    %END;
    %ELSE %DO;
    	data _lab;
        set &COHORTCODES.(obs=0);
        RawGroup=Group;
        keep Code CodeCat Codetype RawGroup;* RawLabResult RawLabDateType Caresettingprincipal;* RawCaresetting;
        run;
    %END;

*Creating the number of groups to create cohorts;
    %isdata(dataset=infolder.&COHORTFILE.);

    %global Ngroup;
    data _null_;
    call symputx("Ngroup","&nobs.");
    run;

*Determine if the chart variable exists in MSDD enrollment table;
    %global chartvar charts;
    data _null_;
    dset=open("indata.&ENRTABLE.");
    call symput ('chartvar',put(varnum(dset,'CHART'),best.));
    run;
    %put &chartvar.;

/************************/
/* HDPS Integrity Check */
/************************/

	%if (%UPCASE("&ANALYSIS.") eq "PS") %then %do ;
		%let message=;

		*Get groups that are not compared with a predefined model;
		data _NotPredefined;
		set infolder.&COMPARISON.(rename=comp=group where=(upcase(HDPS) ne "N")) 
		    infolder.&COMPARISON.(rename=control=group where=(upcase(HDPS) ne "N"));
		keep group HDPSWin: HDPS;
		run;

    proc means data=_NotPredefined nway noprint;
    var HDPSWinfrom HDPSWinTo;
    output out=HDPS_WIN min(HDPSWinfrom)= 
                        max(HDPSWinTo)=;
    run;

    %global MinHDPSWinfrom MaxHDPSWinTo;
    data _NULL_;
    set HDPS_WIN;
    call symputx("MinHDPSWinfrom",put(HDPSWinfrom,best.));
    call symputx("MaxHDPSWinTo",put(HDPSWinTo,best.));
    run;

    %put &MinHDPSWinfrom. &MaxHDPSWinTo.;

		proc sort nodupkey data=_NotPredefined;
		by group;
		run;

		*Create dataset to id which group for HDPS;
		data HDPSSettings;
		set _NotPredefined(in=b);
		by group;
		drop HDPSWin:;
		run;

	%end;


	%global HDPSGROUPS;
	%let HDPSGROUPS=;
	%ISDATA(dataset=infolder.&comparison.);	
	%IF %EVAL(&NOBS.>0) %THEN %DO;
		
		data HDPS;
		set infolder.&comparison.;
		where upcase(HDPS)="Y";
		format group $100.;
		if control ne "" then do;
			group=control;
			output;
		end;
		if comp ne "" then do;
			group=comp;
			output;
		end;
		keep group HDPSWinFrom HDPSWinTo;
		run;

		proc sort nodupkey data=HDPS;
		by group HDPSWinFrom  HDPSWinTo;
		run;

		*check if the same group is there more than once (then it is associated with mon than one HDPS window);
		proc means data=HDPS nway noprint;
		var  HDPSWinFrom  HDPSWinTo;
		class group;
		output out=HDPS min(HDPSWinFrom)=
		                 max(HDPSWinTo)=;
		run;

		%let HDPSPROB=0;
		data HDPS;
		set HDPS;
		if _FREQ_>1 then call symput("HDPSPROB","1");
		run;
		%put &HDPSPROB.;


		%IF %EVAL(&HDPSPROB. = 1) %THEN %DO;
			%put WARNING: The same group is associated with more than one HDPS Window;
		%END;

	%END;

%put NOTE: ******** END OF MACRO: ms_processInputfiles v1.9 ********;

%mend ms_processInputfiles;
