/***************************************************************************************************
*                                          SENTINEL PROGRAM
****************************************************************************************************
* NAME: cder_mpl2p_wp001_public_v01.sas 
*
* PURPOSE: To investigate the risk of stroke among new users of noncyclic combined oral 
*          contraceptives compared to new users of cyclic combined oral contraceptives. 
*
* MAJOR STEPS: See Routine Querying System documentation and request workplan
*
* KEY DEPENDENCIES/CONSTRAINTS/CAVEATS: None
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
* HISTORY:
*  Create date (mm/dd/yy): 01/05/15 
*  Last modified date (mm/dd/yy): 03/01/16 
*  Version: v3.0
*
*  CHANGE LOG: 
*
*   Version   Date       Comment (reference external documentation when available)
*   -------   --------   -------------------------------------------------------------------------
*   2.0       01/05/15   
*
**************************************************************************************************/

/* System options */
options nosymbolgen nomlogic ;
options ls=100 nocenter ;

/*-------------------------------------------------------------------------------------------------
*                                        DEFINITIONS
*--------------------------------------------------------------------------------------------------
* MSCC     ... Sentinel Common-Components File 
*              Specific to each Data Partner/Data Site, the Common-Components defines key
*                 - macro variables (e.g. Data Partner ID, Min and Max Dates, SCDM table names)   
*                 - libname assignments (e.g. libref INDATA is assigned to production SCDM tables) 
*              NOTE: This parameter needs to be set by programmer at the Data Partner/Data Site
*
* MSReqID  ... Sentinel Project Request-Id
*              NOTE: This parameter needs to be set by the programmer preparing the Request-ID
*                    in coordination with the Sentinel Operations Center (SOC)
*
* The following paths for the current request are automatically derived 
*    DPLOCAL  ... path for data created that remains local (i.e behind the DP firewall)
*    MSOC     ... path for data created to share with Sentinel Operations Center
*    INFOLDER ... path for program inputs (e.g.lookup data, external macros)
*    SASPROGRAMS ... path for all SAS programs that are part of the data request
*-------------------------------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* NOTE: Data Partner/Data Site programmer defines parameters in this section */
/*----------------------------------------------------------------------------*/

/* Define Common Components Include File */
/* Example: //sentinel/common-components/ms_common_components.sas */
%let MSCC = FILEPATH/common_components/ms_common_components.sas ;

/* Edit this parameter to identify the list of patients to exclude (if applicable).
   Dataset should contain one variable, patid, and list all patids that should be
   excluded. Must include libname (e.g., indata.PtsToExclude).
 */
%let PTSTOEXCLUDE=;

/* Edit this parameter to make it point to the folder containing dplocal datasets
   from the prior runs (Prior DPLocal) 
*/
%let DPLPrior=;   * Enter the path to the prior run data; 

**************************************************************************************************
*******                                  END OF USER INPUT                                  ******
*******                             DO NOT EDIT BELOW THIS LINE                             ******
*******               (Consult with SOC team leader if you feel edits are required)         ******
**************************************************************************************************;

/* Include Common Components */
%inc "&MSCC" /source2 ;

/*---------------------------------------------------------------------*/
/* NOTE: Programmer preparing Request-Id defines the                   */
/*       parameter MSReqID in coordination with the MSOC               */
/*---------------------------------------------------------------------*/

/*-------------------------------------------------------------------------------------------------
* Specific requests are made up a project-id, workplan-type, workplan-id, unique-dpid, version-id           
*--------------------------------------------------------------------------------------------------
*
*        If this is your request   . . . . .  then set parameter values as follows 
*   ---------------------------------     -------------------------------------------------------
*   [Task Order 7 - NME Dabigatran]       %let MSProjID = to07_nme_dabi ;  
*   [WorkPlan Type]                       %let MSWPType = ahr; 
*   [Workplan # 5]                        %let MSWPID = wp005 ; 
*   [Unique DP id]                        %let MSDPID = nsdp ;     
*   [Beta 3]                              %let MSVerID = b03 ;         
*   [use underscores as delimiter]        %let dlm = _ ;                
*
*   The example values above would produce a Request-Id equal to to07_nme_dabi_ahr_wp005_nsdp_b03
*
*--------------------------------------------------------------------------------------------------*/

* Specify project-id, workplan-type, workplan-id, workplan-type, unique-dpid, version-id *;
%let MSProjID = cder ; 
%let MSWPType = mpl2p ;      
%let MSWPID   = wp001 ;        
%let MSDPID   = nsdp ;   /* Note: set value to nsdp for non-specific DP type requests */
%let MSVerID  = v01 ;        
%let dlm = _ ; 

* Request-Id *;
%let MSReqID = &MSProjID.&dlm.&MSWPType.&dlm.&MSWPID.&dlm.&MSDPID.&dlm.&MSVerID ;

* Delete macro variable used as delimiter as we no longer need it *;
%symdel dlm ; 

*-------------------------------------------------------------------------------------------------
* 0- Setup Environment
*-------------------------------------------------------------------------------------------------;

/*---------------------------------------------------------------------*/
/* NOTE: This is standard MSOC environment setup code -- Do Not Edit   */
/*---------------------------------------------------------------------*/

/* System options */
options obs=MAX ;
options msglevel=i ;
options mprint mprintnest ;
options errorcheck=strict errors=0 ;
options merror serror ;
options dkricond=error dkrocond=error mergenoby=warn;
options dsoptions=nonote2err noquotelenmax ;
options reuse=no ;
options fullstimer ;
options validvarname = v7;

/*------------------------------------------------------------------------------------------------
* NOTES:
*   For reasons of compatibility and standardization, paths must meet the following criteria:
*     1- Starting with the request subdirectory, all subdirectory names must be specified in
*        lower case (for UNIX compatibility)
*     2- Slashes must be forward-leaning slashes '/' (for UNIX compatibility)
*     3- Paths must end with a final forward-leaning slash '/' (extra slashes are fine)
*------------------------------------------------------------------------------------------------*/

/* Request-Id parameter value must be lower case */
%let MSReqID = %lowcase(&MSReqID) ;

/* Define Request Specific Subdirectories */
%let DPLOCAL = &_root_dplocal.&MSReqID./dplocal/ ; 
%let MSOC = &_root_msoc.&MSReqID./msoc/ ;            
%let INFOLDER = &_root_inputfiles.&MSReqID./inputfiles/ ;  
%let SASPROGRAMS = &_root_sasprograms.&MSReqID./sasprograms/ ;

/* Clean up paths (e.g. convert slashes and add slashes when necessary) and validate that paths exist */
%let DPLOCAL = %ms_clean_path(&DPLOCAL) ;
%let MSOC = %ms_clean_path(&MSOC) ;
%let INFOLDER = %ms_clean_path(&INFOLDER) ;
%let SASPROGRAMS = %ms_clean_path(&SASPROGRAMS) ;

/* Assign libnames */
/* %ms_lib(libref-name, path-name, options=%str(options-name=options-value)) */
%ms_lib(dplocal, &DPLOCAL) 
%ms_lib(msoc, &MSOC) 
%ms_lib(infolder, &INFOLDER, options=%str(access=readonly));


*STATLOG;
/********** FOLDER CONTAINING INPUT SAS MACROS USED BY THE MAIN PROGRAM *************************/;
/*Macro Files Location*/          %let sasmacr=&INFOLDER./macros/;			


**************************************************************************************************
*******                                  BEGIN PROGRAM                                     *******
**************************************************************************************************;

/*---------------------------------------------------------------------*/
/* NOTE: Programmer preparing request includes/writes                  */
/*       the rest of the program below                                 */
/*---------------------------------------------------------------------*/

*--------------------------------------------------------------------------------------------------
* 1- 
*--------------------------------------------------------------------------------------------------;


*Added for Routine Querying System;
**Include macro and program files ** ;  
%include "&sasmacr.combo.sas";
%include "&sasmacr.ms_agestrat.sas";
%include "&sasmacr.ms_attrition.sas";
%include "&sasmacr.ms_caresettingprincipal.sas";
%include "&sasmacr.ms_cci_elix.sas";
%include "&sasmacr.ms_cidacov.sas";
%include "&sasmacr.ms_cidadenom.sas";
%include "&sasmacr.ms_cidanum.sas";
%include "&sasmacr.ms_cidatables.sas";
%include "&sasmacr.ms_cidatablest3.sas";
%include "&sasmacr.ms_cidatablest4.sas";
%include "&sasmacr.ms_createcat.sas";
%include "&sasmacr.ms_createclaimepi.sas";
%include "&sasmacr.ms_createclaimepibycat.sas";
%include "&sasmacr.ms_createcontrolgroup.sas";
%include "&sasmacr.ms_createriskset.sas";
%include "&sasmacr.ms_debug.sas";
%include "&sasmacr.ms_delpatients.sas";
%include "&sasmacr.ms_envelope.sas";
%include "&sasmacr.ms_episoderec.sas";
%include "&sasmacr.ms_episoderec2.sas";
%include "&sasmacr.ms_extractdeaths.sas";
%include "&sasmacr.ms_extractdrugs.sas";
%include "&sasmacr.ms_extractlabs.sas";
%include "&sasmacr.ms_extractmeds.sas";
%include "&sasmacr.ms_findgap.sas";
%include "&sasmacr.ms_freezedata.sas";
%include "&sasmacr.ms_loopmeds.sas";
%include "&sasmacr.ms_macros.sas";
%include "&sasmacr.ms_periodsoverlap.sas";
%include "&sasmacr.ms_processinputfiles.sas";
%include "&sasmacr.ms_processwildcards.sas";
%include "&sasmacr.ms_shaveinside.sas";
%include "&sasmacr.ms_shaveoutside.sas";
%include "&sasmacr.ms_stockpiling.sas";
%include "&sasmacr.ms_surveillance.sas";
%include "&sasmacr.ms_time.sas";
%include "&sasmacr.ms_util.sas";

%include "&sasmacr.ms_covariate_adjustment.sas";
%include "&sasmacr.matchtables.sas";

%include "&sasmacr.ms_nearestneighbormatch.sas";
%include "&sasmacr.ms_appendfiles.sas";
%include "&sasmacr.ms_hdps.sas";
%include "&sasmacr.ms_runmodel.sas";

%include "&infolder.run_programs.sas";
